from Homevee.APIModule import APIModule
from Homevee.Item.ShoppingListItem import ShoppingListItem
from Homevee.Item.Status import *
from Homevee.Manager.ShoppingListManager import ShoppingListManager

ACTION_KEY_GET_SHOPPING_LIST = "getshoppinglist"
ACTION_KEY_ADD_EDIT_SHOPPING_LIST_ITEM = "addeditshoppinglistitem"
ACTION_KEY_DELETE_SHOPPING_LIST_ITEM = "deleteshoppinglistitem"

class ShoppingListAPIModule(APIModule):
    def __init__(self):
        super(ShoppingListAPIModule, self).__init__()
        self.shopping_list_manager = ShoppingListManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_SHOPPING_LIST: self.get_shopping_list,
            ACTION_KEY_ADD_EDIT_SHOPPING_LIST_ITEM: self.add_edit_shopping_list_item,
            ACTION_KEY_DELETE_SHOPPING_LIST_ITEM: self.delete_shopping_list_item
        }

        return mappings

    def get_shopping_list(self, user, request, db):
        data = self.shopping_list_manager.get_shopping_list(user, db)
        return Status(type=STATUS_OK, data={'items': ShoppingListItem.list_to_dict(data)})

    def add_edit_shopping_list_item(self, user, request, db):
        data = self.shopping_list_manager.add_edit_shopping_list_item(user, request['id'],
                                                                       request['amount'], request['name'], db)
        return Status(type=STATUS_OK, data=data)

    def delete_shopping_list_item(self, user, request, db):
        data = self.shopping_list_manager.delete_shopping_list_item(user, request['id'], db)
        return Status(type=STATUS_OK, data=data)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.UserManager import UserManager

ACTION_KEY_GET_USERS = "getusers"
ACTION_KEY_ADD_EDIT_USER = "addedituser"
ACTION_KEY_DELETE_USER = "deleteuser"
ACTION_KEY_UPDATE_FCM_TOKEN = "updatefcmtoken"

class UserAPIModule(APIModule):
    def __init__(self):
        super(UserAPIModule, self).__init__()
        self.user_manager = UserManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_USERS: self.get_users,
            ACTION_KEY_ADD_EDIT_USER: self.add_edit_user,
            ACTION_KEY_DELETE_USER: self.delete_user,
            ACTION_KEY_UPDATE_FCM_TOKEN: self.update_fcm_token
        }

        return mappings

    def get_users(self, user, request, db):
        users = self.user_manager.get_users(user, db)
        return Status(type=STATUS_OK, data={'users':self.user_manager.get_dict_list_from_item_list(users)})

    def add_edit_user(self, user, request, db):
        self.user_manager.add_edit_user(user, request['name'], request['psw'], request['ip'],
                                    request['permissions'], db)
        return Status(type=STATUS_OK)

    def delete_user(self, user, request, db):
        self.user_manager.delete_user(user, request['name'], db)
        return Status(type=STATUS_OK)

    def update_fcm_token(self, user, request, db):
        self.user_manager.set_user_fcm_token(user, request['token'], db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.CalendarEntry import CalendarEntry
from Homevee.Item.Status import *
from Homevee.Manager.CalendarManager import CalendarManager

ACTION_KEY_GET_CALENDAR_ITEM_DATES = "getcalendaritemdates"
ACTION_KEY_GET_CALENDAR_DAY_ITEMS = "getcalendardayitems"
ACTION_KEY_ADD_EDIT_CALENDAR_ENTRY = "addeditcalendarentry"
ACTION_KEY_DELETE_CALENDAR_ENTRY = "deletecalendarentry"

class CalendarAPIModule(APIModule):
    def __init__(self):
        super(CalendarAPIModule, self).__init__()
        self.calendar_manager = CalendarManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_CALENDAR_ITEM_DATES: self.get_calendar_item_dates,
            ACTION_KEY_GET_CALENDAR_DAY_ITEMS: self.get_calendar_day_items,
            ACTION_KEY_ADD_EDIT_CALENDAR_ENTRY: self.add_edit_entry,
            ACTION_KEY_DELETE_CALENDAR_ENTRY: self.delete_entry
        }

        return mappings

    def get_calendar_item_dates(self, user, request, db):
        dates = self.calendar_manager.get_calendar_item_dates(user, request['year'], db)
        return Status(type=STATUS_OK, data={'dates':dates})

    def get_calendar_day_items(self, user, request, db):
        items = self.calendar_manager.get_calendar_day_items(user, request['date'], db)
        return Status(type=STATUS_OK, data={'items': CalendarEntry.list_to_dict(items)})

    def add_edit_entry(self, user, request, db):
        self.calendar_manager.add_edit_entry(user, request['id'], request['name'], request['date'],
                                                        request['start'],
                                                        request['end'], request['note'], request['address'], db)
        return Status(type=STATUS_OK)

    def delete_entry(self, user, request, db):
        if self.calendar_manager.delete_entry(user, request['id'], db):
            return Status(type=STATUS_OK)
        else:
            return Status(type=STATUS_ERROR)


import json
import traceback

from Homevee.APIModule import APIModule
from Homevee.Exception import *
from Homevee.Helper import translations, Logger
from Homevee.Item.Status import *
from Homevee.VoiceAssistant import VoiceAssistant
from Homevee.VoiceAssistant.VoiceReplaceManager import VoiceReplaceManager

ACTION_KEY_VOICE_COMMAND = "voicecommand"
ACTION_KEY_GET_VOICE_REPLACE_ITEMS = "getvoicereplaceitems"
ACTION_KEY_ADD_EDIT_VOICE_REPLACE_ITEM = "addeditvoicereplaceitem"
ACTION_KEY_DELETE_VOICE_REPLACE_ITEM = "deletevoicereplaceitem"

class VoiceAPIModule(APIModule):
    def __init__(self):
        super(VoiceAPIModule, self).__init__()
        self.voice_replace_manager = VoiceReplaceManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_VOICE_COMMAND: self.voice_command,
            ACTION_KEY_GET_VOICE_REPLACE_ITEMS: self.get_voice_replace_items,
            ACTION_KEY_ADD_EDIT_VOICE_REPLACE_ITEM: self.add_edit_voice_replace_item,
            ACTION_KEY_DELETE_VOICE_REPLACE_ITEM: self.delete_voice_replace_item
        }
        return mappings

    def voice_command(self, user, request, db) -> Status:
        if 'language' in request:
            language = request['language']
        else:
            language = translations.LANGUAGE

        if ('user_last_location' in request and request['user_last_location'] is not None):
            user_last_location = json.loads(request['user_last_location'])
        else:
            user_last_location = None

        try:
            response = VoiceAssistant().do_voice_command(user, request['text'], user_last_location, None, db, language)
            return Status(type=STATUS_OK, data=response)
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return Status(type=STATUS_ERROR)

    def get_voice_replace_items(self, user, request, db) -> Status:
        try:
            replace_data = self.voice_replace_manager.get_voice_replace_items(user, db)
        except(NoPermissionException):
            return Status(type=STATUS_NO_PERMISSION)
        except(DatabaseSaveFailedException):
            return Status(type=STATUS_ERROR)
        return Status(type=STATUS_OK, data={'replacedata': replace_data})

    def add_edit_voice_replace_item(self, user, request, db) -> Status:
        try:
            success = self.voice_replace_manager.add_edit_voice_replace_item(user, request['replacewith'], request['itemstoreplace'], db)
            if success:
                return Status(type=STATUS_OK)
            else:
                return Status(type=STATUS_ERROR)
        except(NoPermissionException):
            return Status(type=STATUS_NO_PERMISSION)
        except:
            return Status(type=STATUS_ERROR)

    def delete_voice_replace_item(self, user, request, db) -> Status:
        try:
            success = self.voice_replace_manager.delete_voice_replace_item(user, request['replacewith'], db)
            if success:
                return Status(type=STATUS_OK)
            else:
                return Status(type=STATUS_ERROR)
        except(NoPermissionException):
            return Status(type=STATUS_NO_PERMISSION)
        except:
            return Status(type=STATUS_ERROR)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Device import Device
from Homevee.Item.Device.Thermostat.MaxThermostat import *
from Homevee.Item.Device.Thermostat.RademacherThermostat import *
from Homevee.Item.Device.Thermostat.ZWaveThermostat import *
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Utils.DeviceTypes import *

class ThermostatManager:
    def __init__(self):
        return

    def heating_control(self, user, type, id, value, db, check_user=True):
        if type == MAX_CUBE:
            module = MaxThermostat
        elif type == ZWAVE_THERMOSTAT:
            module = ZWaveThermostat
        elif type == RADEMACHER_THERMOSTAT:
            module = RademacherThermostat
        else:
            return Status(type=STATUS_NO_SUCH_TYPE).get_dict()

        thermostat = Device.load_from_db(module, id)

        if check_user and not user.has_permission(thermostat.location):
            return Status(type=STATUS_NO_PERMISSION).get_dict()

        if(thermostat.set_temp(value)):
            return Status(type=STATUS_OK).get_dict()
        else:
            return Status(type=STATUS_ERROR).get_dict()

    def control_room_heating(self, user, room, value, db: Database = None):
        if db is None:
            db = Database()
            devices = self.get_thermostats(user, room, db)[0]['thermostat_array']
            for device in devices:
                id = device['id']
                type = device['type']
                result = self.heating_control(user, type, id, value, db)
                if 'result' not in result or result['result'] != "ok":
                    return Status(type=STATUS_ERROR).get_dict()
            return Status(type=STATUS_OK).get_dict()

    def get_thermostat_info(self, user, room, type, id, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(room):
            return Status(type=STATUS_NO_PERMISSION).get_dict()

        module_map = {
            ZWAVE_THERMOSTAT: ZWaveThermostat,
            MAX_THERMOSTAT: MaxThermostat,
            RADEMACHER_THERMOSTAT: RademacherThermostat
        }

        if type not in module_map:
            return Status(type=STATUS_NO_SUCH_TYPE).get_dict()

        thermostat = Device.load_from_db(module_map[type], id)

        min, max = thermostat.get_min_max()
        return {'value': thermostat.temp, 'min': min, 'max': max}

    def get_thermostats(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        if isinstance(room, Room):
            rooms = [room]
        elif room == "all":
            rooms = Room.load_all(db)
        else:
            rooms = Room.load_all_ids_from_db([room])

        thermostats = []

        for room in rooms:
            if not user.has_permission(room.id):
                continue

            thermostat_array = []

            modules = [ZWaveThermostat, MaxThermostat, RademacherThermostat]

            for module in modules:
                thermostat_items = Device.get_all(module, room)
                for thermostat in thermostat_items:
                    min, max = thermostat.get_min_max()
                    thermostat_data = {'name': thermostat.name, 'id': thermostat.id, 'type': thermostat.get_device_type(),
                                       'icon': thermostat.icon, 'data': {'value': thermostat.temp, 'min': min, 'max': max}}
                    thermostat_array.append(thermostat_data)

            room_thermostats = {'name': room.name, 'location': room.id, 'icon': room.icon,
                                'thermostat_array': thermostat_array}
            thermostats.append(room_thermostats)

        return thermostats



#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI import zwave
from Homevee.Exception import NoSuchTypeException, NoPermissionException
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *

class DimmerManager:
    def __init__(self):
        return

    def get_dimmers(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(room):
            return Status(type=STATUS_NO_PERMISSION).get_dict()

        dimmer = []
        #Z-Wave Dimmer
        results = db.select_all("SELECT * FROM ZWAVE_DIMMER WHERE LOCATION == :room", {'room': room})

        for item in results:
            dimmer_item = {'device': item['ID'], 'value': item['VALUE'], 'icon': item['ICON'],
                         'name': item['NAME'], 'type': ZWAVE_DIMMER}
            dimmer.append(dimmer_item)

        return dimmer

    def set_dimmer(self, user, type, id, value, db: Database = None):
        if db is None:
            db = Database()
        if type == ZWAVE_DIMMER:
            return self.set_zwave_dimmer(user, id, value, db)
        else:
            raise NoSuchTypeException

    def set_zwave_dimmer(self, user, id, value, db: Database = None):
        if db is None:
            db = Database()

            data = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID == :device",
                        {'device': id})

            if not user.has_permission(data['LOCATION']):
                raise NoPermissionException

            device_id = data['ID']

            result = zwave.device_control.set_multistate_device(device_id, value, db)

            if result['code'] == 200:
                db.update("UPDATE ZWAVE_DIMMER SET VALUE = :value WHERE ID = :id",
                            {'value': value, 'id': id})
            else:
                raise Exception


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.RFIDTagManager import RFIDTagManager

ACTION_KEY_GET_RFID_TAGS = "getrfidtags"
ACTION_KEY_ADD_EDIT_RFID_TAG = "addeditrfidtag"
ACTION_KEY_DELETE_RFID_TAG = "deleterfidtag"
ACTION_KEY_RUN_RFID_ACTION = "runrfidaction"

class RFIDTagAPIModule(APIModule):
    def __init__(self):
        super(RFIDTagAPIModule, self).__init__()
        self.rfid_tag_manager = RFIDTagManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_RFID_TAGS: self.get_rfid_tags,
            ACTION_KEY_ADD_EDIT_RFID_TAG: self.add_edit_rfid_tag,
            ACTION_KEY_DELETE_RFID_TAG: self.delete_rfid_tag,
            ACTION_KEY_RUN_RFID_ACTION: self.run_frid_action
        }

        return mappings

    def get_rfid_tags(self, user, request, db):
        data = self.rfid_tag_manager.get_rfid_tags(user, db)
        return Status(type=STATUS_OK, data={'tags': data})

    def add_edit_rfid_tag(self, user, request, db):
        data = self.rfid_tag_manager.add_edit_rfid_tag(user, request['name'], request['uuid'], request['type'],
                                                          request['id'], db)
        return Status(type=STATUS_OK, data=data)

    def delete_rfid_tag(self, user, request, db):
        data = self.rfid_tag_manager.delete_rfid_tag(user, request['uuid'], db)
        return Status(type=STATUS_OK, data=data)

    def run_frid_action(self, user, request, db):
        data = self.rfid_tag_manager.run_rfid_action(user, request['uuid'], db)
        return Status(type=STATUS_OK, data=data)


from Homevee.AIAgents.ImageClassifier import ImageClassifier


class PeopleClassifier(ImageClassifier):
    def __init__(self):
        model_name = "people_temp"
        super(PeopleClassifier, self).__init__(model_name)
        return


#!/usr/bin/python
# -*- coding: utf-8 -*-

import time

from Homevee.Item.Event import Event
from Homevee.Utils.Database import Database


class EventManager():
    def __init__(self):
        return

    '''Gibt die Ereignisse zurück'''
    def get_events(self, user, type, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        events = Event.load_all_from_db_desc_date_by_type(offset, limit, type)
        user.events_last_checked = time.time()
        user.save(db)
        return events

    '''Gibt die Anzahl der ungesehenen Ereignisse zurück'''
    def get_unseen_events(self, user, db: Database = None):
        if db is None:
            db = Database()
        count = Event.get_unseen_events(user)
        return count

    '''Gibt die vorhandenen Ereignis-Typen zurück'''
    def get_event_types(self, db):
        results = db.select_all("SELECT DISTINCT TYPE FROM EVENTS ORDER BY TYPE", {})

        types = []

        for data in results:
            types.append(data['TYPE'])

        return types

    '''Erstellt ein neues Ereignis'''
    def add_event(self, type, text, db: Database = None):
        if db is None:
            db = Database()
        event = Event(text=text, type=type)
        return event.save()


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.ActionManager import ActionManager
from Homevee.Manager.SceneManager import SceneManager

ACTION_KEY_GET_ALL_SCENES = "getallscenes"
ACTION_KEY_GET_SCENES = "getscenes"
ACTION_KEY_ADD_EDIT_SCENE = "addeditscene"
ACTION_KEY_DELETE_SCENE = "deletescene"
ACTION_KEY_RUN_SCENE = "runscene"

class SceneAPIModule(APIModule):
    def __init__(self):
        super(SceneAPIModule, self).__init__()
        self.scenes_manager = SceneManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_ALL_SCENES: self.get_all_scenes,
            ACTION_KEY_GET_SCENES: self.get_scenes,
            ACTION_KEY_ADD_EDIT_SCENE: self.add_edit_scene,
            ACTION_KEY_DELETE_SCENE: self.delete_scene,
            ACTION_KEY_RUN_SCENE: self.run_scene
        }

        return mappings

    def get_all_scenes(self, user, request, db):
        data = self.scenes_manager.get_all_scenes(user, db)
        return Status(type=STATUS_OK, data={'scenes': data})

    def get_scenes(self, user, request, db):
        data = self.scenes_manager.get_scenes(user, request['location'], db)
        return Status(type=STATUS_OK, data={'scenes': data})

    def add_edit_scene(self, user, request, db):
        self.scenes_manager.add_edit_scene(user, request['id'], request['name'], request['location'],
                                                     request['action_data'], db)
        return Status(type=STATUS_OK)

    def delete_scene(self, user, request, db):
        self.scenes_manager.delete_scene(user, request['id'], db)
        return Status(type=STATUS_OK)

    def run_scene(self, user, request, db):
        ActionManager().run_scene(user, request['id'], db)
        return Status(type=STATUS_OK)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item import Item
from Homevee.Item.ShoppingListItem import ShoppingListItem
from Homevee.Utils.Database import Database

class ShoppingListManager:
    def __init__(self):
        return

    def get_shopping_list(self, user, db: Database = None):
        """
        Gets all shoppinglist-items from the database
        :param user: the calling user
        :param db: the databse connection
        :return: the list of dicts of items on the shopping list
        """
        if db is None:
            db = Database()
        items = ShoppingListItem.load_all(db)
        return items

    def add_edit_shopping_list_item(self, user, id, amount, name, db: Database = None):
        """
        Adds a new item to the shopping list
        :param user: the calling user
        :param id: the id of the item
        :param amount: the amount of the item
        :param name: the name of the item
        :param db: the database connection
        :return: the dict of the status object of the transaction
        """
        if db is None:
            db = Database()
        shopping_list_item = Item.load_from_db(ShoppingListItem, id, db)
        if shopping_list_item is None:
            shopping_list_item = ShoppingListItem(name, amount, id)
        else:
            shopping_list_item.amount = amount
            shopping_list_item.name = name

        return shopping_list_item.save(db)

    def delete_shopping_list_item(self, user, id, db: Database = None):
        """
        Deletes an item from the shopping list
        :param user: the calling user
        :param id: the id of the item to delete
        :param db: the database connection
        :return: the dict of status object of the transaction
        """
        if db is None:
            db = Database()
        item = Item.load_from_db(ShoppingListItem, id, db)
        return item.delete(db)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.GPSDataManager import GPSDataManager

ACTION_KEY_UPDATE_GPS = "updategps"
ACTION_KEY_GET_GPS_LOCATIONS = "getgpslocations"

class GPSDataAPIModule(APIModule):
    def __init__(self):
        super(GPSDataAPIModule, self).__init__()
        self.gps_data_manager = GPSDataManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_UPDATE_GPS: self.update_gps,
            ACTION_KEY_GET_GPS_LOCATIONS: self.get_gps_locations
        }

        return mappings

    def update_gps(self, user, request, db):
        self.gps_data_manager.update_gps(user, request['time'], request['lat'], request['lng'], db)
        return Status(type=STATUS_OK)

    def get_gps_locations(self, user, request, db):
        locations = self.gps_data_manager.get_gps_locations(user, db)
        return Status(type=STATUS_OK, data={'locations':locations})


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.PlaceManager import PlaceManager

ACTION_KEY_ADD_EDIT_PLACE = "addeditplace"
ACTION_KEY_GET_MY_PLACES = "getmyplaces"
ACTION_KEY_DELETE_PLACE = "deleteplace"

class PlaceAPIModule(APIModule):
    def __init__(self):
        super(PlaceAPIModule, self).__init__()
        self.place_manager = PlaceManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_ADD_EDIT_PLACE: self.add_edit_place,
            ACTION_KEY_GET_MY_PLACES: self.get_my_places,
            ACTION_KEY_DELETE_PLACE: self.delete_place
        }

        return mappings

    def add_edit_place(self, user, request, db):
        self.place_manager.add_edit_place(user, request['id'], request['name'],
                                                     request['address'], request['latitude'],
                                                     request['longitude'], db)
        return Status(type=STATUS_OK)

    def get_my_places(self, user, request, db):
        places = self.place_manager.get_my_places(user, db)
        return Status(type=STATUS_OK, data={'places':places})

    def delete_place(self, user, request, db):
        self.place_manager.delete_place(user, request['id'], db)
        return Status(type=STATUS_OK)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.DeviceAPI.set_modes import set_modes
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.BlindsManager import BlindsManager
from Homevee.Manager.ControlManager.RGBLightManager import RGBLightManager
from Homevee.Manager.ControlManager.ThermostatManager import ThermostatManager
from Homevee.Manager.ControlManager.WakeOnLanManager import WakeOnLanManager
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *
from Homevee.Utils.NotificationManager import NotificationManager


class ActionManager():
    def __init__(self):
        return

    def run_scene(self, user, id, db: Database = None):
        if db is None:
            db = Database()

        result = db.select_one("SELECT * FROM SCENES WHERE ID = :id",
                    {'id': id})

        if (result is None):
            return None

        id = result['ID']

        action_data = result['ACTION_DATA']

        action_data = json.loads(action_data)

        # run actions
        self.run_actions(user, action_data)

        return Status(type=STATUS_OK).get_dict()

    def run_actions(self, user, action_data, db: Database = None):
        if db is None:
            db = Database()
        for action in action_data:
            if action['type'] == "push_notification":
                msg = action['message']
                users = action['users']
                NotificationManager().send_notification_to_users(users, "Homevee", msg)
            elif action['type'] == "run_scene":
                scene_id = action['id']
                self.run_scene(user, scene_id)
            elif action['type'] == 'control_device':
                device_type = action['devicetype']
                device_id = action['id']

                if device_type in [FUNKSTECKDOSE, URL_TOGGLE, URL_SWITCH, ZWAVE_SWITCH]:
                    set_modes(user, device_type, device_id, action['value'], db, False)
                elif device_type == WAKE_ON_LAN:
                    WakeOnLanManager().wake_on_lan(user, device_id, db, False)
                elif device_type == XBOX_ONE_WOL:
                    WakeOnLanManager().wake_xbox_on_lan(user, device_id, db, False)
                elif device_type in [ZWAVE_THERMOSTAT, MAX_THERMOSTAT, RADEMACHER_THERMOSTAT]:
                    ThermostatManager().heating_control(user, device_type, device_id, action['value'], db, False)
                elif device_type in [RADEMACHER_BLIND_CONTROL]:
                    BlindsManager().set_blinds(user, device_type, device_id, action['value'], db, False)
                elif device_type in [PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]:
                    data = json.loads(action['value'])
                    RGBLightManager().rgb_control(user, device_type, device_id, data['mode'], data['brightness'], data['color'], db, False)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import traceback

from Homevee.Helper import Logger
from Homevee.Helper.HomeveeAPI import HomeveeAPI
from Homevee.Utils.Database import Database


class TVPlanManager:
    def __init__(self):
        return

    def get_tv_plan(self, user, type, db: Database = None):
        if db is None:
            db = Database()
        try:
            tv_channels = self.get_tv_channels(user)

            if type == "2015":
                type = "heute2015"
            elif type == "2200":
                type = "heute2200"
            elif type == "jetzt":
                type = "jetzt"
            elif type == "tipps":
                type = "tipps"
            else:
                raise AttributeError("TV-Typ '" + type + "' nicht vorhanden")

            tv_shows = []

            response = HomeveeAPI().get_tv_data(type)

            if(response.status_code == 200):
                tv_data = json.loads(response.response)['tvprogramm']
                for item in tv_data:
                    if type == "tipps" or ((tv_channels is None) or (item['channel'] in tv_channels)):
                        tv_shows.append(item)
                return tv_shows
            else:
                return None
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return None

    def get_tv_channels(self, user, db: Database = None):
        if db is None:
            db = Database()

        results = db.select_all("SELECT * FROM TV_CHANNELS WHERE USERNAME == :username", {'username': user.username})

        channels = []

        for channel in results:
            channels.append(channel['CHANNEL'])

        return channels

    def get_all_tv_channels(self, user, db: Database = None):
        if db is None:
            db = Database()
        selected_channels = self.get_tv_channels(user)

        response = HomeveeAPI().get_tv_data("channels")

        if(response.status_code == 200):
            all_channels = json.loads(response.response)
            print(all_channels)

            channels = []
            for channel in all_channels['channels']:
                if (channels is None) or (channel not in channels):
                    channels.append({'name': channel, 'selected': (channel in selected_channels)})

            return channels
        else:
            return None

    def set_tv_channels(self, user, json_data, db: Database = None):
        if db is None:
            db = Database()
        result = db.delete("DELETE FROM TV_CHANNELS WHERE USERNAME == :username", {'username': user.username})

        #Abfrage erfolgreich?
        if result:
            channels = json.loads(json_data)

            for channel in channels:
                db.insert("INSERT INTO TV_CHANNELS (USERNAME, CHANNEL) VALUES (?,?)",
                    [user.username, channel])
        else:
            raise Exception


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database import Database

class TriggerManager:
    def __init__(self):
        return

    def add_trigger(self, user, location, id, db: Database = None):
        if db is None:
            db = Database()
        return

    def get_triggers(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(room):
            return {'result': 'nopermission'}

        triggers = []

        results = db.select_all("SELECT * FROM MQTT_TRIGGERS WHERE LOCATION = :location", {'location': room})

        for item in results:
            triggers.append({'name': item['NAME'], 'id': item['ID'], 'type': 'MQTT-Trigger', 'icon': item['ICON']})

        return triggers


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.NutritionDataManager import NutritionDataManager

ACTION_KEY_SUBMIT_FOOD = "submitfood"
ACTION_KEY_GET_USER_NUTRITION_OVERVIEW = "getusernutritionoverview"
ACTION_KEY_GET_USER_DAY_NUTRITION_ITEMS = "getuserdaynutritionitems"
ACTION_KEY_ADD_EDIT_USER_DAY_NUTRITION_ITEMS = "addedituserdaynutritionitem"
ACTION_KEY_LOAD_NUTRITION_MANAGER_SETTINGS = "loadnutritionmanagersettings"
ACTION_KEY_DELETE_FOOD_ITEM = "deletefooditem"
ACTION_KEY_SAVE_NUTRITION_MANAGER_SETTINGS = "savenutritionmanagersettings"
ACTION_KEY_MOVE_NUTRTITION_ITEM = "movenutritionitem"

class NutritionDataAPIModule(APIModule):
    def __init__(self):
        super(NutritionDataAPIModule, self).__init__()
        self.nutrition_data_manager = NutritionDataManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_SUBMIT_FOOD: self.submit_food,
            ACTION_KEY_GET_USER_NUTRITION_OVERVIEW: self.get_user_nutrition_overview,
            ACTION_KEY_GET_USER_DAY_NUTRITION_ITEMS: self.get_user_day_nutrition_items,
            ACTION_KEY_ADD_EDIT_USER_DAY_NUTRITION_ITEMS: self.add_edit_user_day_nutrition_item,
            ACTION_KEY_LOAD_NUTRITION_MANAGER_SETTINGS: self.load_user_settings,
            ACTION_KEY_DELETE_FOOD_ITEM: self.delete_food_item,
            ACTION_KEY_SAVE_NUTRITION_MANAGER_SETTINGS: self.save_user_settings,
            ACTION_KEY_MOVE_NUTRTITION_ITEM: self.move_item
        }

        return mappings

    def submit_food(self, user, request, db):
        success = self.nutrition_data_manager.submit_food(user, request['name'], request['calories'],
                                                          request['portionsize'],
                                                          request['portionunit'], request['protein'], request['fat'],
                                                          request['saturated'], request['unsaturated'],
                                                          request['carbs'],
                                                          request['sugar'], request['ean'], db)
        if success:
            return Status(type=STATUS_OK)
        else:
            return Status(type=STATUS_ERROR)

    def get_user_nutrition_overview(self, user, request, db):
        data = self.nutrition_data_manager.get_user_nutrition_overview(user, db)
        return Status(type=STATUS_OK, data=data)

    def get_user_day_nutrition_items(self, user, request, db):
        data = self.nutrition_data_manager.get_user_day_nutrition_items(user, request['date'], db)
        return Status(type=STATUS_OK, data=data)

    def add_edit_user_day_nutrition_item(self, user, request, db):
        self.nutrition_data_manager.add_edit_user_day_nutrition_item(user, request['id'], request['date'],
                                                                               request['daytime'], request['name'],
                                                                               request['eatenportionsize'],
                                                                               request['portionsize'],
                                                                               request['portionunit'],
                                                                               request['calories'], request['fat'],
                                                                               request['saturated'],
                                                                               request['unsaturated'],
                                                                               request['carbs'], request['sugar'],
                                                                               request['protein'], db)
        return Status(type=STATUS_OK)

    def load_user_settings(self, user, request, db):
        data = self.nutrition_data_manager.load_user_settings(user, db)
        return Status(type=STATUS_OK, data=data)

    def delete_food_item(self, user, request, db):
        self.nutrition_data_manager.delete_food_item(user, request['id'], db)
        return Status(type=STATUS_OK)

    def save_user_settings(self, user, request, db):
        data = self.nutrition_data_manager.save_user_settings(user, request['height'], request['weight'],
                                                                 request['birthdate'],
                                                                 request['mode'], request['activity'], db)
        return Status(type=STATUS_OK, data=data)

    def move_item(self, user, request, db):
        self.nutrition_data_manager.move_item(user, request['id'], request['date'], request['daytime'],
                                                        request['deleteold'], db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.RGBLightManager import RGBLightManager

ACTION_KEY_SET_RGB = "setrgb"

class RGBLightAPIModule(APIModule):
    def __init__(self):
        super(RGBLightAPIModule, self).__init__()
        self.rgb_manager = RGBLightManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_SET_RGB: self.set_rgb
        }

        return mappings

    def set_rgb(self, user, request, db):
        self.rgb_manager.rgb_control(user, request['type'], request['id'], request['mode'],
                                                     request['brightness'],
                                                     request['color'], db)
        return Status(type=STATUS_OK)


from Homevee import Homevee
from Homevee.Manager.UserManager import UserManager
from Homevee.Utils.Database import Database


class TestDataGenerator:
    def __init__(self):
        self.db = Database()
        return

    def generate_test_data(self):
        self.clear_database()

        Homevee().add_user(username="testadmin", password="testpw", is_admin=True)

        admin_user = UserManager().fin

        self.generate_data()

    def clear_database(self):
        pass

    def generate_test_data(self):
        pass


import os

from Homevee.DeviceAPI.XBOXOneAPI import XBOXOneAPI
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database


class WakeOnLanManager:
    def __init__(self):
        return

    def wake_on_lan(self, user, id, db, check_user=True):
        device = db.select_all("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})
        if check_user and not user.has_permission(device['LOCATION']):
            return {'result': 'nopermission'}

        os.system("sudo wakeonlan "+device['MAC_ADDRESS'])

    def get_wol_devices(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(room):
            return Status(type=STATUS_NO_PERMISSION).get_dict()

        devices = []

        results = db.select_all("SELECT * FROM WAKE_ON_LAN WHERE LOCATION == :room", {'room': room})

        for data in results:
            devices.append({'name': data['NAME'], 'id': data['DEVICE'], 'icon': data['ICON']})

        return devices

    def wake_xbox_on_lan(self, user, id, db, check_user=True):
            device = db.select_all("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id",
                        {'id': id})

            if check_user and not user.has_permission(device['LOCATION']):
                return {'result': 'nopermission'}

            ip = device['IP_ADDRESS']
            live_id = device['XBOX_LIVE_ID']

            XBOXOneAPI().xbox_wake_up(ip, live_id)

    def get_xbox_devices(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(room):
            return Status(type=STATUS_NO_PERMISSION).get_dict()

        devices = []
        results = db.select_all("SELECT * FROM XBOX_ONE_WOL WHERE LOCATION == :room", {'room': room})

        for data in results:
            devices.append({'name': data['NAME'], 'id': data['ID'], 'icon': data['ICON']})

        return devices


from Homevee.AIAgents.ImageClassifier import ImageClassifier


class ARControlClassifier(ImageClassifier):
    def __init__(self):
        model_name = "ar_control_temp"
        super(ARControlClassifier, self).__init__(model_name)
        return


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json

import requests
from dateutil.relativedelta import relativedelta

from Homevee.Helper import Logger
from Homevee.Manager.RemoteControlManager import RemoteControlManager
from Homevee.Utils.Database import Database


class NutritionDataManager:
    def __init__(self):
        return

    def get_nutrition_data(self, user, keyword, db: Database = None):
        if db is None:
            db = Database()
        data = {'action': 'findsingleitem', 'term': keyword}

        Logger.log("Nutrition item search: "+keyword)

        r = requests.post("https://food.Homevee.de/api.php", data=data)
        # Logger.log(r.status_code, r.reason)

        if (r.status_code != 200):
            return None

        response = r.text

        #print response

        data = json.loads(response)

        if(data is None):
            return None

        if('food' not in data):
            return None

        nutrition_data = data['food']

        if(nutrition_data is None):
            return None

        if(len(nutrition_data) is 0):
            return False

        return nutrition_data[0]

    def submit_food(self, user, name, calories, portionsize, portionunit, protein, fat, saturated, unsaturated, carbs, sugar, ean, db: Database = None):
        if db is None:
            db = Database()
        remote_data = RemoteControlManager().load_remote_data(user)
        remote_id = remote_data['remote_id']

        if(remote_id is None or remote_id == ""):
            return {'result': 'noremoteid'}

        data = {'action': 'submitfood', 'name': name, 'calories': calories, 'portionsize': portionsize, 'portionunit': portionunit,
                'protein': protein, 'fat': fat, 'saturated': saturated, 'unsaturated': unsaturated, 'carbs': carbs,
                'sugar': sugar, 'ean': ean, 'user_remote_id': remote_id}

        r = requests.post("https://food.Homevee.de/api.php", data=data)
        #Logger.log(r.status_code, r.reason)

        if(r.status_code != 200):
            return {"result": "error"}

        result = json.loads(r.text)

        if("result" in result):
            if(result['result'] == "ok"):
                return True
        return False

    def get_user_day_nutrition_data(self, user, db: Database = None):
        if db is None:
            db = Database()
        return

    def get_user_nutrition_overview(self, user, db: Database = None):
        if db is None:
            db = Database()
        user_profile = self.get_user_fitness_profile(user)

        Logger.log(user_profile)

        output = {}

        if (user_profile is False):
            has_profile = False
        else:
            has_profile = True

            results = db.select_all("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                        {'user': user.username, 'date': datetime.datetime.today().strftime("%d.%m.%Y")})

            eaten_calories_today = 0
            eaten_fat_today = 0
            eaten_carbs_today = 0
            eaten_protein_today = 0

            for item in results:
                amount = (float(item['EATEN_PORTION_SIZE'])/float(item['PORTIONSIZE']))

                eaten_calories_today += item['CALORIES']*amount
                eaten_fat_today += item['FAT']*amount
                eaten_carbs_today += item['CARBS']*amount
                eaten_protein_today += item['PROTEIN']*amount

            #print datetime.datetime.today().strftime("%Y-%m-%d")

            nutrition_distribution = {'fat': eaten_fat_today, 'carbs': eaten_carbs_today,
                                      'protein': eaten_protein_today}
            nutrition_day_data = {'calories_goal': int(user_profile['caloriesgoal']),
                                  'calories_left': int(user_profile['caloriesgoal']-eaten_calories_today),
                                  'nutrition_distribution': nutrition_distribution}

            output['nutrition_day_data'] = nutrition_day_data

        weight_progress = {}

        output['has_profile'] = has_profile
        output['weight_progress'] = weight_progress

        return output

    def save_user_settings(self, user, height, weight, birthdate, mode, activity, db: Database = None):
        if db is None:
            db = Database()
        calories_goal = self.get_calorie_need(birthdate, weight, height, activity, mode)

        protein_goal, carbs_goal, fat_goal = self.get_macro_distribution(calories_goal)

        water_goal = 2.0

        self.add_user_weight(user, weight)
        result = db.select_one("SELECT COUNT(*) FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

        result = result['COUNT(*)']

        param_array = {'cals': calories_goal, 'user': user.username, 'fat': fat_goal, 'saturated': 0, 'carbs': carbs_goal,
                       'sugar': 0, 'protein': protein_goal, 'water': water_goal, 'birthdate': birthdate, 'height': height,
                       'weight': weight, 'activity':activity, 'diet': mode}

        if(result == 0):
            db.insert("INSERT INTO USER_FITNESS_DATA (user, CALORIE_GOAL, FAT_GOAL, SATURATED_GOAL, CARBS_GOAL, SUGAR_GOAL, PROTEIN_GOAL, WATER_GOAL, BIRTHDATE, HEIGHT, ACTIVITY_MODE, DIET_MODE) VALUES (:user, :cals, :fat, :saturated, :carbs, :sugar, :protein, :water, :birthdate, :height, :activity, :diet)", param_array)
        else:
            db.update("UPDATE USER_FITNESS_DATA SET CALORIE_GOAL = :cals, FAT_GOAL = :fat, SATURATED_GOAL = :saturated, CARBS_GOAL = :carbs, SUGAR_GOAL = :sugar, PROTEIN_GOAL = :protein, WATER_GOAL = :water, BIRTHDATE = :birthdate, HEIGHT = :height, ACTIVITY_MODE = :activity, DIET_MODE = :diet WHERE USERNAME = :user", param_array)

        return {'result': 'ok', 'calories_goal': calories_goal, 'protein_goal': protein_goal, 'carbs_goal': carbs_goal, 'fat_goal': fat_goal}

    def get_macro_distribution(self, calories):

        protein_factor = 0.25
        carbs_factor = 0.5
        fat_factor = 0.25

        protein_goal = (calories/4)*protein_factor
        carbs_goal = (calories/4)*carbs_factor
        fat_goal = (calories/8)*fat_factor

        return int(protein_goal), int(carbs_goal), int(fat_goal)

    def load_user_settings(self, user, db: Database = None):
        if db is None:
            db = Database()
        data = {}

        #{'sitting': 1.2, 'lightactivity': 1.35, 'moderateactivity': 1.55, 'highactivity': 1.75, 'extremeactivity': 1.95}

        data['activities'] = [
            {'title': 'Sitzend', 'description': 'wenig/kein Training + Bürotätigkeit', 'key': 'sitting'},
            {'title': 'leichte Aktivität', 'description': 'geringe tägliche Aktivität + leichtes Training 1-3 Mal/ Woche', 'key': 'lightactivity'},
            {'title': 'moderate Aktivität', 'description': 'moderate tägliche Aktivität + moderates Training 3-5 Mal/ Woche', 'key': 'moderateactivity'},
            {'title': 'hohe Aktivität', 'description': 'körperlich anspruchsvoller Lifestyle + schweres Training/Sport 6-7 Mal/ Woche', 'key': 'highactivity'},
            {'title': 'extrem hohe Aktivität', 'description': 'hartes tägliches Training/Sport und/oder körperlich beanspruchender Job', 'key': 'extremeactivity'}
        ]

        data['modes'] = [
            {'title': 'stark Abnehmen', 'key': 'harddiet'},
            {'title': 'Abnehmen', 'key': 'diet'},
            {'title': 'Gewicht halten', 'key': 'hold'},
            {'title': 'Muskeln aufbauen', 'key': 'gainmuscles'}
        ]

        result = db.select_one("SELECT * FROM USER_WEIGHT_DATA WHERE USERNAME = :user ORDER BY DATE DESC", {'user': user.username})
        weight_data = result['WEIGHT']

        result = db.select_one("SELECT * FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

        activity_title = None
        for item in data['activities']:
            if(item['key'] == result['ACTIVITY_MODE']):
                activity_title = item['title']
                break

        mode_title = None
        for item in data['modes']:
            if(item['key'] == result['DIET_MODE']):
                mode_title = item['title']
                break

        settings = {'weight': weight_data, 'height': result['HEIGHT'], 'birthdate': result['BIRTHDATE'],'activity': result['ACTIVITY_MODE'],
                    'activity_title': activity_title, 'mode': result['DIET_MODE'], 'mode_title': mode_title}

        data['settings'] = settings

        return data

    def add_user_weight(self, user, weight, db: Database = None):
        if db is None:
            db = Database()
        date = datetime.datetime.today().__format__("%Y-%m-%d")

        db.insert("INSERT INTO USER_WEIGHT_DATA (USERNAME, DATE, WEIGHT) VALUES (:user, :date, :weight)",
                   {'user': user.username, 'date': date, 'weight': weight})

    def delete_food_item(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        db.delete("DELETE FROM NUTRITION_DATA WHERE ID = :id AND USER = :user",
                    {'id': id, 'user': user.username})

    def get_calorie_need(self, birthdate, weight, height, movement_mode, diet_mode):
        # (9.99 x Körpergewicht (in kg)) + (6.25 x Körpergröße (in cm)) – (4.92 x Alter (in Jahren)) – 161

        birthdate = datetime.datetime.strptime(birthdate, "%Y-%m-%d")
        today = datetime.datetime.today()
        age = relativedelta(today, birthdate).years

        base_calories = (9.99 * int(weight)) + (6.25 * int(height)) - (4.92 * age) - 161

        movement_factor = 1.2
        movement_map = {'sitting': 1.2, 'lightactivity': 1.35, 'moderateactivity': 1.55, 'highactivity': 1.75, 'extremeactivity': 1.95}
        if(movement_mode in movement_map):
            movement_factor = movement_map[movement_mode]
        calories_with_movement = base_calories * movement_factor

        diet_factor = 1
        diet_map = {'harddiet': 0.8, 'diet': 0.9, 'hold': 1.0, 'musclegain': 1.1}
        if(diet_mode in diet_map):
            diet_factor = diet_map[diet_mode]
        Logger.log(diet_mode)
        diet_calories = calories_with_movement * diet_factor

        calories_needed = diet_calories

        return int(calories_needed)

    def get_user_fitness_profile(self, user, db: Database = None):
        if db is None:
            db = Database()

        result = db.select_one("SELECT * FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

        if(result is None):
            return False

        data = {'caloriesgoal': result['CALORIE_GOAL'], 'fatgoal': result['FAT_GOAL'], 'saturatedgoal': result['SATURATED_GOAL'],
                'carbsgoal': result['CARBS_GOAL'], 'sugargoal': result['SUGAR_GOAL'], 'proteingoal': result['PROTEIN_GOAL'],
                'watergoal': result['WATER_GOAL']}

        return data

    def get_user_weight_progress(self, user, start, end, db: Database = None):
        if db is None:
            db = Database()
        start = datetime.datetime.strptime(start, "%d.%m.%Y").strftime("%Y-%m-%d")
        end = datetime.datetime.strptime(end, "%d.%m.%Y").strftime("%Y-%m-%d")

        weight_items = []

        results = db.select_all("SELECT WEIGHT, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM USER_WEIGHT_DATA WHERE USERNAME = :user AND DATE >= :start AND DATE <= :ende ORDER BY DATE ASC",
                    {'user': user.username, 'start': start, 'end': end})

        for item in results:
            weight_items.append({'date': item['DATETIME'], 'weight': item['WEIGHT']})

        return weight_items

    def get_user_day_nutrition_items(self, user, date, db: Database = None):
        if db is None:
            db = Database()
        daytime_items = {'morning': [], "noon": [], "evening": [], "snacks": []}

        results = db.select_all("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                    {'user': user.username, 'date': date})

        for item in results:
            food_item = {'id': item['ID'], 'daytime': item['DAYTIME'], 'name': item['FOOD_NAME'], 'eatenportionsize': item['EATEN_PORTION_SIZE'],
                         'portionsize': item['PORTIONSIZE'], 'portionunit': item['PORTIONUNIT'], 'calories': item['CALORIES'],
                         'fat': item['FAT'], 'saturated': item['SATURATED'], 'unsaturated': item['UNSATURATED'],
                         'carbs': item['CARBS'], 'sugar': item['SUGAR'], 'protein': item['PROTEIN'], 'date': item['DATE']}

            if(item['DAYTIME'] in daytime_items):
                daytime_items[item['DAYTIME']].append(food_item)
            else:
                daytime_items['snacks'].append(food_item)

        calories_goal = self.get_user_fitness_profile(user)['caloriesgoal']

        return {'items': daytime_items, 'caloriesgoal': calories_goal}

    def move_item(self, user, id, date, daytime, deleteold, db: Database = None):
        if db is None:
            db = Database()
        item = db.select_all("SELECT * FROM NUTRITION_DATA WHERE ID = :id", {'id': id})

        if(item is None):
            return {'result': 'itemnotexisting'}

        name = item['FOOD_NAME']
        eatenportionsize = item['EATEN_PORTION_SIZE']
        portionsize = item['PORTIONSIZE']
        portionunit = item['PORTIONUNIT']
        calories = item['CALORIES']
        fat = item['FAT']
        saturated = item['SATURATED']
        unsaturated = item['UNSATURATED']
        carbs = item['CARBS']
        sugar = item['SUGAR']
        protein = item['PROTEIN']

        self.add_edit_user_day_nutrition_item(user, None, date, daytime, name, eatenportionsize, portionsize, portionunit,
                                         calories, fat, saturated, unsaturated, carbs, sugar, protein)

        if deleteold == "true" or deleteold == True or deleteold == 1 or deleteold == "1":
            db.delete("DELETE FROM NUTRITION_DATA WHERE ID = :id", {'id': id})

    def add_edit_user_day_nutrition_item(self, user, id, date, daytime, name, eatenportionsize, portionsize, portionunit, calories, fat, saturated, unsaturated, carbs, sugar, protein, db: Database = None):
        if db is None:
            db = Database()
        add_new = (id == None or id == "" or id == "-1")

        param_array = {'user': user.username, 'date': date, 'daytime': daytime, 'name': name, 'eatenportionsize': eatenportionsize,
                       'portionsize': portionsize, 'portionunit': portionunit, 'calories': calories, 'fat': fat,
                       'saturated': saturated, 'unsaturated': unsaturated, 'carbs': carbs, 'sugar': sugar, 'protein': protein}

        if add_new:
            db.insert("INSERT INTO NUTRITION_DATA (USER, DATE, DAYTIME, FOOD_NAME, EATEN_PORTION_SIZE, PORTIONSIZE, PORTIONUNIT, CALORIES, FAT, SATURATED, UNSATURATED, CARBS, SUGAR, PROTEIN) VALUES (:user, :date, :daytime, :name, :eatenportionsize, :portionsize, :portionunit, :calories, :fat, :saturated, :unsaturated, :carbs, :sugar, :protein)",
                        param_array)
        else:
            param_array['id'] = id

            db.update("UPDATE NUTRITION_DATA SET USER = :user, DATE = :date, DAYTIME = :daytime, FOOD_NAME = :name, EATEN_PORTION_SIZE = :eatenportionsize, PORTIONSIZE = :portionsize, PORTIONUNIT = :portionunit, CALORIES = :calories, FAT = :fat, SATURATED = :saturated, UNSATURATED = :unsaturated, CARBS = :carbs, SUGAR = :sugar, PROTEIN = :protein WHERE ID = :id",
                        param_array)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.SensorDataManager import SensorDataManager

ACTION_KEY_GET_SENSOR_DATA = "getsensordata"

class SensorDataAPIModule(APIModule):
    def __init__(self):
        super(SensorDataAPIModule, self).__init__()
        self.sensor_data_manager = SensorDataManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_SENSOR_DATA: self.get_sensor_data
        }

        return mappings

    def get_sensor_data(self, user, request, db):
        data = self.sensor_data_manager.get_sensor_data(user, request['room'], db)
        return Status(type=STATUS_OK, data={'values': data})


from Homevee.AIAgents.ImageClassifier.ARControlClassifier import ARControlClassifier
from Homevee.AIAgents.ImageClassifierRetrainer.ARControlRetrainer import ARControlRetrainer
from Homevee.Manager.ControlManager.ImageClassifierManager import ImageClassifierManager


class ARControlManager(ImageClassifierManager):
    def __init__(self):
        data_table = "AR_CONTROL_CLASSES"
        learning_data_table = "AR_CONTROL_LEARNING_DATA"
        settings_prefix = "PEOPLE_"

        classifier = ARControlClassifier()
        retrainer = ARControlRetrainer()

        is_training_tag = "AR_CONTROL_TRAINING_RUNNING"
        classifier_tag = "arcontrol"

        super(ARControlManager, self).__init__(data_table=data_table, learning_data_table=learning_data_table,
                                                      settings_prefix=settings_prefix, is_training_tag=is_training_tag,
                                                      classifier_tag=classifier_tag, classifier=classifier, retrainer=retrainer)
        return


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Updater import Updater

ACTION_KEY_CHECK_FOR_UPDATES = "checkforupdates"
ACTION_KEY_UPDATE_SYSTEM = "updatesystem"

class UpdaterAPIModule(APIModule):
    def __init__(self):
        super(UpdaterAPIModule, self).__init__()
        self.update_manager = Updater()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_CHECK_FOR_UPDATES: self.check_for_updates,
            ACTION_KEY_UPDATE_SYSTEM: self.update_system
        }

        return mappings

    def check_for_updates(self, user, request, db):
        data = self.update_manager.check_for_updates()
        return Status(type=STATUS_OK, data=data)

    def update_system(self, user, request, db):
        self.update_manager.do_homevee_update(user, db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.HeatingSchemeManager import HeatingSchemeManager

ACTION_KEY_ADD_EDIT_HEATING_SCHEME_ITEM = "addeditheatingschemeitem"
ACTION_KEY_DELETE_HEATING_SCHEME_ITEM = "deleteheatingschemeitem"
ACTION_KEY_GET_HEATING_SCHEME_ITEMS = "getheatingschemeitems"
ACTION_KEY_SET_HEATING_SCHEME_ITEM_ACTIVE = "setheatingschemeitemactive"
ACTION_KEY_GET_HEATING_SCHEME_ITEM_DATA = "getheatingschemeitemdata"
ACTION_KEY_SET_HEATING_SCHEME_ACTIVE = "setheatingschemeactive"
ACTION_KEY_IS_HEATING_SCHEME_ACTIVE = "isheatingschemeactive"

class HeatingSchemeAPIModule(APIModule):
    def __init__(self):
        super(HeatingSchemeAPIModule, self).__init__()
        self.heating_scheme_manager = HeatingSchemeManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_ADD_EDIT_HEATING_SCHEME_ITEM: self.add_edit_heating_scheme_item,
            ACTION_KEY_DELETE_HEATING_SCHEME_ITEM: self.delete_heating_scheme_item,
            ACTION_KEY_GET_HEATING_SCHEME_ITEMS: self.get_heating_scheme_items,
            ACTION_KEY_SET_HEATING_SCHEME_ITEM_ACTIVE: self.set_heating_scheme_item_active,
            ACTION_KEY_GET_HEATING_SCHEME_ITEM_DATA: self.get_heating_scheme_item_data,
            ACTION_KEY_SET_HEATING_SCHEME_ACTIVE: self.set_heating_scheme_active,
            ACTION_KEY_IS_HEATING_SCHEME_ACTIVE: self.is_heating_scheme_active
        }

        return mappings

    def add_edit_heating_scheme_item(self, user, request, db):
        data = self.heating_scheme_manager.add_edit_heating_scheme_item(user, request['id'], request['time'],
                                                                           request['value'],
                                                                           request['active'], request['days'],
                                                                           request['data'], db)
        return Status(type=STATUS_OK, data=data)

    def delete_heating_scheme_item(self, user, request, db):
        data = self.heating_scheme_manager.delete_heating_scheme_item(user, request['id'], db)
        return Status(type=STATUS_OK, data=data)

    def get_heating_scheme_items(self, user, request, db):
        data = self.heating_scheme_manager.get_heating_scheme_items(user, request['day'], request['rooms'], db)
        return Status(type=STATUS_OK, data={'items': data})

    def set_heating_scheme_item_active(self, user, request, db):
        self.heating_scheme_manager.set_heating_scheme_item_active(user, request['id'], request['active'], db)
        return Status(type=STATUS_OK)

    def get_heating_scheme_item_data(self, user, request, db):
        data = self.heating_scheme_manager.get_heating_scheme_item_data(user, request['id'], db)
        return Status(type=STATUS_OK, data=data)

    def set_heating_scheme_active(self, user, request, db):
        self.heating_scheme_manager.set_heating_scheme_active(user, request['active'], db)
        return Status(type=STATUS_OK)

    def is_heating_scheme_active(self, user, request, db):
        data = self.heating_scheme_manager.is_heating_scheme_active(user, db)
        return Status(type=STATUS_OK, data={'isactive': data})


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request

from Homevee.DeviceAPI import kodi_api
from Homevee.DeviceAPI.kodi_api import KodiAPI
from Homevee.Exception import NoSuchTypeException
from Homevee.Helper import Logger
from Homevee.Utils.Database import Database

TYPE_KODI = "KODI"

class MediaCenterManager:
    def __init__(self):
        self.kodi_api = KodiAPI()
        return

    def get_media_centers(self, user, db: Database = None):
        if db is None:
            db = Database()
        media_centers = []

        results = db.select_all("SELECT *, MEDIA_CENTER.NAME as MEDIA_CENTER_NAME, ROOMS.NAME as ROOM_NAME FROM MEDIA_CENTER, ROOMS WHERE ROOMS.LOCATION = MEDIA_CENTER.LOCATION", {})

        for result in results:
            if(user.has_permission(result['LOCATION'])):
                item = {'name': result['MEDIA_CENTER_NAME'], 'type': result['TYPE'], 'id': result['ID'],
                        'location': result['LOCATION'], 'value': result['ROOM_NAME']}
                media_centers.append(item)

        return media_centers

    def media_remote_action(self, type, id, remoteaction, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            return self.kodi_api.media_remote_action(self.get_data(id), remoteaction)

    def send_text(self, type, id, text, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            return self.kodi_api.send_text(self.get_data(id), text)

        raise NoSuchTypeException

    def get_music(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_music(self.get_data(id), limit, offset)

            result = json.loads(result)

            songs = []

            song_array = result['result']['songs']

            for song in song_array:
                item = {'title': song['label'], 'artists': song['artist'], 'id': song['songid'], 'album': song['album']}
                songs.append(item)

            return songs

        raise NoSuchTypeException

    def get_artists(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_artists(self.get_data(id), limit, offset)

            result = json.loads(result)

            artists = []

            artist_array = result['result']['artists']

            for artist in artist_array:
                item = {'artist': artist['artist'], 'id': artist['artistid'], 'thumbnail': artist['thumbnail']}
                artists.append(item)

            return artists

        raise NoSuchTypeException

    def get_albums(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_albums(self.get_data(id), limit, offset)

            result = json.loads(result)

            albums = []

            album_array = result['result']['albums']

            for album in album_array:
                item = {'title': album['label'], 'artists': album['artist'], 'id': album['albumid'], 'thumbnail': album['thumbnail']}
                albums.append(item)

            return albums

        raise NoSuchTypeException

    def get_music_genres(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_music_genres(self.get_data(id), limit, offset)

            result = json.loads(result)

            genres = []

            genre_array = result['result']['genres']

            for genre in genre_array:
                item = {'title': genre['label'], 'id': genre['genreid'], 'thumbnail': genre['thumbnail']}
                genres.append(item)

            return genres

        raise NoSuchTypeException

    def get_movie_genres(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_movie_genres(self.get_data(id), limit, offset)

            Logger.log(result)

            result = json.loads(result)

            genres = []

            genre_array = result['result']['genres']

            for genre in genre_array:
                item = {'title': genre['label'], 'id': genre['genreid'], 'thumbnail': genre['thumbnail']}
                genres.append(item)

            return genres

        raise NoSuchTypeException

    def get_shows(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_tv_show(self.get_data(id), limit, offset)

            result = json.loads(result)

            tvshows = []

            tvshow_array = result['result']['tvshows']

            for tvshow in tvshow_array:

                thumbnail = tvshow['thumbnail']
                thumbnail = thumbnail.replace('image://', '')
                thumbnail = urllib.parse.unquote(thumbnail).decode('utf8')

                item = {'title': tvshow['label'], 'genres': tvshow['genre'], 'id': tvshow['tvshowid'],
                        'year': ['year'], 'thumbnail': thumbnail}
                tvshows.append(item)

            return tvshows

        raise NoSuchTypeException

    def get_show_seasons(self, type, id, limit, offset, showid, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_tv_show_seasons(self.get_data(id), showid, limit, offset)

            result = json.loads(result)

            seasons = []

            season_array = result['result']['tvshows']

            for season in season_array:

                thumbnail = season['thumbnail']
                thumbnail = thumbnail.replace('image://', '')
                thumbnail = urllib.parse.unquote(thumbnail).decode('utf8')

                item = {'title': season['label'], 'id': season['seasonid'], 'thumbnail': thumbnail}
                seasons.append(item)

            return seasons

        raise NoSuchTypeException

    def get_show_episodes(self, type, id, limit, offset, showid, seasonid, db: Database = None):
        if db is None:
            db = Database()
        return []

    def get_movies(self, type, id, limit, offset, db: Database = None):
        if db is None:
            db = Database()
        if type == TYPE_KODI:
            result = self.kodi_api.get_movies(self.get_data(id), limit, offset)

            result = json.loads(result)

            movies = []

            movie_array = result['result']['movies']

            for movie in movie_array:

                thumbnail = movie['thumbnail']
                thumbnail = thumbnail.replace('image://', '')
                thumbnail = urllib.parse.unquote(thumbnail).decode('utf8')

                item = {'title': movie['label'], 'genres': movie['genre'], 'id': movie['movieid'],
                        'year': movie['year'], 'thumbnail': thumbnail}
                movies.append(item)

            return movies

        raise NoSuchTypeException

    def get_playing(self, type, id, db: Database = None):
        if db is None:
            db = Database()
        return []

    def get_data(self, id, db: Database = None):
        if db is None:
            db = Database()
        result = db.select_one("SELECT * FROM MEDIA_CENTER WHERE ID == :id", {'id': id})
        return result


import operator
import os

import numpy as np

from Homevee.Utils import Constants

try:
    import tensorflow as tf
    TENSORFLOW_INSTALLED = True
    #Logger.log("Tensorflow imported")
except:
    TENSORFLOW_INSTALLED = False
    #Logger.log("Error importing Tensorflow")

# These are all parameters that are tied to the particular model architecture
# we're using for Inception v3. These include things like tensor names and their
# sizes. If you want to adapt this script to work with another model, you will
# need to update these to reflect the values in the network you're using.
MAX_NUM_IMAGES_PER_CLASS = 2 ** 27 - 1  # ~134M

class ImageClassifier():
    def __init__(self, model_name):
        self.model_name = model_name

        return

    def load_graph(self, model_file):
        graph = tf.Graph()
        graph_def = tf.GraphDef()
        with open(model_file, "rb") as f:
            graph_def.ParseFromString(f.read())
        with graph.as_default():
            tf.import_graph_def(graph_def)
        return graph

    def read_tensor_from_image_file(self, file_name, input_height=299, input_width=299,
                                    input_mean=0, input_std=255):
        input_name = "file_reader"
        output_name = "normalized"
        file_reader = tf.read_file(file_name, input_name)
        if file_name.endswith(".png"):
            image_reader = tf.image.decode_png(file_reader, channels=3, name='png_reader')
        elif file_name.endswith(".gif"):
            image_reader = tf.squeeze(tf.image.decode_gif(file_reader, name='gif_reader'))
        elif file_name.endswith(".bmp"):
            image_reader = tf.image.decode_bmp(file_reader, name='bmp_reader')
        else:
            image_reader = tf.image.decode_jpeg(file_reader, channels=3, name='jpeg_reader')
        float_caster = tf.cast(image_reader, tf.float32)
        dims_expander = tf.expand_dims(float_caster, 0)
        resized = tf.image.resize_bilinear(dims_expander, [input_height, input_width])
        normalized = tf.divide(tf.subtract(resized, [input_mean]), [input_std])
        sess = tf.Session()
        result = sess.run(normalized)
        return result

    def load_labels(self, label_file):
        label = []
        proto_as_ascii_lines = tf.gfile.GFile(label_file).readlines()
        for l in proto_as_ascii_lines:
            label.append(l.rstrip())
        return label

    def predict(self, file_name):
        file_name = os.path.join(Constants.DATA_DIR, file_name)
        BASE_DIR = os.path.dirname(os.path.abspath(__file__))
        model_dir = os.path.join(BASE_DIR, "tensorflow_data", "models", "mobilenet_v1_1.0_224")
        temp_dir = os.path.join(BASE_DIR, "tensorflow_data", self.model_name)
        model_file = os.path.join(temp_dir, "output_graph.pb")
        label_file = os.path.join(temp_dir, "output_labels.txt")
        input_height = 224
        input_width = 224
        input_mean = 0
        input_std = 255
        input_layer = "input"
        output_layer = "final_result"
        '''if args.graph:
            model_file = args.graph
        if args.labels:
            label_file = args.labels
        if args.input_layer:
            input_layer = args.input_layer
        if args.output_layer:
            output_layer = args.output_layer'''
        graph = self.load_graph(model_file)
        t = self.read_tensor_from_image_file(file_name, input_height=input_height, input_width=input_width,
                                        input_mean=input_mean, input_std=input_std)
        input_name = "import/" + input_layer
        output_name = "import/" + output_layer
        input_operation = graph.get_operation_by_name(input_name)
        output_operation = graph.get_operation_by_name(output_name)
        with tf.Session(graph=graph) as sess:
            results = sess.run(output_operation.outputs[0], {input_operation.outputs[0]: t})
        results = np.squeeze(results)
        top_k = results.argsort()[-5:][::-1]
        labels = self.load_labels(label_file)
        max_val = -1
        max_label = -1
        predictions = []
        for i in top_k:
            predictions.append({'prediction': labels[i], 'confidence': results[i] * 100})
            if (results[i] > max_val):
                max_val = results[i]
                max_label = labels[i]
        # sorting descending by confidence
        sorted_predictions = sorted(predictions, key=operator.itemgetter('confidence'), reverse=True)
        return sorted_predictions


from Homevee.AIAgents.ImageClassifierRetrainer import ImageClassifierRetrainer


class PeopleClassifierRetrainer(ImageClassifierRetrainer):
    def __init__(self):
        image_dir_name = "people_classifier"
        model_dir_name = "people_temp"
        retrainer_name = "Personenerkennung"
        super(PeopleClassifierRetrainer, self).__init__(image_dir_name, model_dir_name, retrainer_name)


import traceback

from Homevee.APIModule import APIModule
from Homevee.Item.ChatMessage import ChatMessage
from Homevee.Item.Status import *
from Homevee.Manager.ChatManager import ChatManager

ACTION_KEY_GET_CHAT_MESSAGES = "getchatmessages"
ACTION_KEY_GET_CHAT_IMAGE = "getchatimage"
ACTION_KEY_SEND_CHAT_MESSAGE = "sendchatmessage"

class ChatAPIModule(APIModule):
    def __init__(self):
        super(ChatAPIModule, self).__init__()
        self.chat_manager = ChatManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_CHAT_MESSAGES: self.get_chat_messages,
            ACTION_KEY_GET_CHAT_IMAGE: self.get_chat_image,
            ACTION_KEY_SEND_CHAT_MESSAGE: self.send_chat_message
        }
        return mappings

    def get_chat_messages(self, user, request, db) -> Status:
        try:
            messages = self.chat_manager.get_chat_messages(user, request['time'], request['limit'], db)
            return Status(type=STATUS_OK, data={'messages': ChatMessage.list_to_dict(messages)})
        except:
            return Status(type=STATUS_ERROR)

    def get_chat_image(self, user, request, db) -> Status:
        try:
            data = self.chat_manager.get_chat_image(user, request['imageid'], db)
            return Status(type=STATUS_OK, data=data)
        except:
            return Status(type=STATUS_ERROR)

    def send_chat_message(self, user, request, db) -> Status:
        try:
            data = self.chat_manager.send_chat_message(user, request['data'], db)
            return Status(type=STATUS_OK, data=data)
        except:
            traceback.print_exc()
            return Status(type=STATUS_ERROR)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Item import Item
from Homevee.Item.HomeBudgetItem import HomeBudgetItem
from Homevee.Utils.Database import Database


class HomeBudgetManager:
    def __init__(self):
        return

    def get_home_budget_data(self, user, startdate, db: Database = None):
        if db is None:
            db = Database()
        data = {}

        #Find expenditures grouped by date
        results = db.select_all("SELECT SUM(AMOUNT) as AMOUNT, strftime(\"%d.%m.%Y\", DATE) as DATE FROM HOME_BUDGET_DATA WHERE AMOUNT <= 0 AND DATE >= :date GROUP BY DATE ORDER BY DATE ASC",
                    {'date': startdate})
        costs = []

        last_date = None

        for item in results:
            this_date = datetime.datetime.strptime(item['DATE'], "%d.%m.%Y")
            if(last_date is not None):
                difference = (this_date - last_date).days

                for i in range(1, difference):
                    date = last_date + datetime.timedelta(days=i)
                    costs.append({'date': date.strftime("%d.%m.%Y"), 'amount': round(0, 2)})

            last_date = this_date

            costs.append({'date': item['DATE'], 'amount': round(item['AMOUNT']*-1, 2)})
        data['costs'] = costs

        #Find amount money left
        item = db.select_one("SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA", {})
        overall_amount = round(item['AMOUNT'], 2)
        data['overall_amount'] = overall_amount

        #Compute date where money reaches 0
        item = db.select_one("SELECT AVG(AMOUNT) as AMOUNT FROM (SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA WHERE AMOUNT < 0 GROUP BY DATE);", {})
        average_spending_per_day = item['AMOUNT']*-1

        #compute timeperiod
        min_max_date = db.select_one("SELECT MIN(DATE) as mindate, MAX(DATE) as maxdate, COUNT(*) as numdays FROM (SELECT DISTINCT DATE FROM HOME_BUDGET_DATA);", {})
        real_numdays = (datetime.datetime.today()-datetime.datetime.strptime(min_max_date['mindate'], "%Y-%m-%d")).days
        average_spending_per_day = average_spending_per_day * min_max_date['numdays'] / real_numdays
        days_left = overall_amount/average_spending_per_day
        date = datetime.datetime.today() + datetime.timedelta(days=days_left)
        date = datetime.datetime.strftime(date, "%d.%m.%Y")
        data['sufficient_until'] = date

        return data

    def get_home_budget_data_graph(self, user, startdate, enddate, db: Database = None):
        if db is None:
            db = Database()
        item = db.select_one("SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA WHERE DATE < :startdate;",
            {'startdate': startdate})

        startvalue = item['AMOUNT']

        if startvalue is None:
            startvalue = 0

        #Get values
        results = db.select_all("SELECT SUM(AMOUNT) as AMOUNT, strftime(\"%d.%m.%Y\", DATE) as FORMATTED_DATE FROM HOME_BUDGET_DATA WHERE DATE >= :startdate AND DATE <= :enddate GROUP BY DATE ORDER BY DATE ASC",
            {'startdate': startdate, 'enddate': enddate})

        values = []

        last_date = None

        for item in results:
            this_date = datetime.datetime.strptime(item['FORMATTED_DATE'], "%d.%m.%Y")
            if (last_date is not None):
                difference = (this_date - last_date).days

                for i in range(1, difference):
                    date = last_date + datetime.timedelta(days=i)
                    values.append({'date': date.strftime("%d.%m.%Y"), 'amount': round(0, 2)})

            last_date = this_date

            values.append({'date': item['FORMATTED_DATE'], 'amount': round(item['AMOUNT'], 2)})

        return {'startvalue': startvalue, 'values': values}

    def get_home_budget_data_day_items(self, user, date, db: Database = None):
        if db is None:
            db = Database()
        data = []

        results = db.select_all("SELECT *, strftime(\"%d.%m.%Y\", DATE) as FORMATTED_DATE FROM HOME_BUDGET_DATA WHERE DATE = :date ORDER BY DATE ASC",
                    {'date': date})

        for item in results:
            data.append({'id': item['ID'], 'date': item['FORMATTED_DATE'], 'amount': round(item['AMOUNT'], 2), 'info': item['INFO']})

        return data

    def add_edit_home_budget_data(self, user, id, date, info, amount, db: Database = None):
        if db is None:
            db = Database()
        home_budget_item = None
        if id is not None:
            home_budget_item = Item.load_from_db(HomeBudgetItem, id, db)
        if home_budget_item is None:
            home_budget_item = HomeBudgetItem(date, info, amount, id)
        else:
            home_budget_item.date = date
            home_budget_item.info = info
            home_budget_item.amount = amount

        return home_budget_item.save(db)

    def delete_home_budget_data(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        item = Item.load_from_db(HomeBudgetItem, id)
        return item.delete(db)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Item.ChatMessage import ChatMessage
from Homevee.Utils.Database import Database

class ChatManager:
    def __init__(self):
        return

    def get_chat_messages(self, user, time, limit, db: Database = None):
        """
        Loads the last X chat messages from the database
        :param user:
        :param time:
        :param limit:
        :param db:
        :return:
        """
        if db is None:
            db = Database()
        messages = ChatMessage.load_all_by_time(time, limit)
        return messages

    def get_chat_image(self, user, imageid, db: Database = None):
        if db is None:
            db = Database()
        return {}

    def send_chat_message(self, user, data, db: Database = None):
        """
        Sends a new chat message to all users
        :param user:
        :param data:
        :param db:
        :return:
        """
        if db is None:
            db = Database()
        message = ChatMessage(user.username, data)
        return message.send(db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import time

from Homevee.Item.Room import Room
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *


class EnergyDataManager:
    def __init__(self):
        return

    def get_power_usage_devices(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        devices = []

        for item in self.get_energy_device_data(user, room, db):
            try:
                if item['value'] is not None:
                    item['value'] = str(round(item['value'], 2)) + " kWh"
                else:
                    item['value'] = "N/A"
            except:
                item['value'] = "N/A"
            devices.append(item)

        return devices

    def get_energy_device_data(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        devices = []

        if not user.has_permission(room):
            return {'result': 'nopermission'}

        # Z-Wave Stromzähler
        results = db.select_all("SELECT * FROM ZWAVE_POWER_METER WHERE ROOM_ID = :room", {'room': room})
        for item in results:
            try:
                value = float(item['VALUE'])
            except:
                value = "N/A"
            devices.append({'name': item['DEVICE_NAME'], 'id': item['DEVICE_ID'], 'devicetype': ZWAVE_POWER_METER,
                            'icon': item['ICON'], 'value': value, 'location': room})
        return devices

    def get_current_power_usage(self, user, db: Database = None):
        if db is None:
            db = Database()
        devices = []

        rooms = Room.load_all(db)

        for room in rooms:
            devices.extend(self.get_energy_device_data(user, room.id, db))

        return devices

    def get_current_room_power_usage(self, user, db: Database = None):
        if db is None:
            db = Database()
        rooms = {}

        for room in Room.load_all(db):
            usage = 0
            for device in self.get_energy_device_data(user, room.id, db):
                if device['value'] is not None and device['value'] != "N/A":
                    usage += float(device['value'])

            rooms[str(room.id)] = {'usage': usage, 'devices': self.get_energy_device_data(user, room.id, db)}

        return rooms

    def get_current_device_power_usage(self, user, room, type, id, db: Database = None):
        if db is None:
            db = Database()
        devices = self.get_power_usage_devices(user, room, db)

        for device in devices:
            if device['devicetype'] == type and device['id'] == id:
                return device['VALUE']
        return None

    def get_energy_data(self, user, room, devicetype, deviceid, von, bis, db: Database = None):
        if db is None:
            db = Database()
        von = datetime.datetime.strptime(von, '%d.%m.%Y')
        bis = datetime.datetime.strptime(bis, '%d.%m.%Y')

        if((room is None or room == "")
                and (devicetype is None or devicetype == "")
                and (deviceid is None or deviceid == "")):

            room_data = self.get_current_room_power_usage(user, db)

            rooms_sum = 0
            for room_item in room_data:
                rooms_sum += room_data[room_item]['usage']

            data = {}

            result = db.select_one("SELECT SUM(POWER_USAGE) AS POWER_USAGE FROM ENERGY_DATA WHERE DATE >= :von AND DATE <= :bis",
                        {'von': von, 'bis': bis})

            data['totalusage'] = 0

            if result['POWER_USAGE'] is not None:
                data['totalusage'] += float(result['POWER_USAGE'])

            if rooms_sum is not None:
                data['totalusage'] += float(rooms_sum)

            data['cost'] = self.get_power_cost(user, db)

            percentage = []
            results = db.select_all("SELECT ROOMS.NAME, SUM(POWER_USAGE) AS USAGE, ROOM_ID FROM ROOM_ENERGY_DATA, ROOMS WHERE ROOM_ID = ROOMS.LOCATION AND DATE >= :von AND DATE <= :bis GROUP BY ROOM_ID",
                        {'von': von, 'bis': bis})

            for item in results:
                energy_item = {'type': '', 'id': '', 'location': item['ROOM_ID'],
                                    'name': item['NAME'], 'usage': float(item['USAGE'])+room_data[str(item['ROOM_ID'])]['usage']}
                percentage.append(energy_item)

            data['usagepercentage'] = percentage

            data['usagecourse'] = self.get_energy_course(user, room, von, bis, db)
            data['usagecourse'].append({'date': time.strftime('%d.%m.%Y'), 'usage': rooms_sum})

            return data
        elif(type is not None and type != '' and deviceid is not None and deviceid != '' and room is not None and room != ''):
            return self.get_device_energy_data(user, room, devicetype, deviceid, von, bis, db)
        else:
            return self.get_room_energy_data(user, room, von, bis, db)

    def get_room_energy_data(self, user, room, von, bis, db: Database = None):
        if db is None:
            db = Database()
        data = {}
        room_data = self.get_current_room_power_usage(user, db)[room]['devices']
        room_data_dict = {}
        room_total_usage = 0
        for item in room_data:
            room_total_usage += item['value']
            room_data_dict[str(item['id'])+str(item['devicetype'])] = item['value']

        result = db.select_one("SELECT SUM(POWER_USAGE) AS POWER_USAGE FROM ROOM_ENERGY_DATA WHERE ROOM_ID = :room AND DATE >= :von AND DATE <= :bis",
                    {'room': room, 'von': von, 'bis': bis})

        power_usage = 0

        if(result['POWER_USAGE'] is not None):
            power_usage = result['POWER_USAGE']

        data['totalusage'] = float(power_usage)+room_total_usage
        data['cost'] = self.get_power_cost(user, db)

        percentage = []

        device_types = []

        device_types.append("SELECT SUM(POWER_USAGE) AS POWER_USAGE, DEVICE_NAME, DEVICE_TYPE, DEVICE_ENERGY_DATA.DEVICE_ID FROM DEVICE_ENERGY_DATA, ZWAVE_POWER_METER WHERE LOCATION = ROOM_ID AND DEVICE_TYPE = \""+ZWAVE_POWER_METER+"\" AND LOCATION = :room AND DATE >= :von AND DATE <= :bis GROUP BY DEVICE_NAME")

        #device_types.append("SELECT SUM(POWER_USAGE) AS POWER_USAGE, DEVICE_NAME, DEVICE_TYPE, DEVICE_ENERGY_DATA.DEVICE_ID	FROM DEVICE_ENERGY_DATA, DIY_POWER_METER WHERE LOCATION = ROOM_ID AND DEVICE_TYPE = "+DIY+" AND LOCATION = :room AND DATE >= :von AND DATE <= :bis GROUP BY DEVICE_NAME")

        #print("SELECT SUM(POWER_USAGE) AS POWER_USAGE, DEVICE_NAME, DEVICE_TYPE, DEVICE_ENERGY_DATA.DEVICE_ID FROM DEVICE_ENERGY_DATA, ZWAVE_POWER_METER WHERE LOCATION = ROOM_ID AND DEVICE_TYPE = \""+ZWAVE_POWER_METER+"\" AND LOCATION = \""+str(room)+"\" AND DATE >= \""+str(von.strftime('%Y-%m-%d'))+"\" AND DATE <= \""+str(bis.strftime('%Y-%m-%d'))+"\" GROUP BY DEVICE_NAME")

        for device_type in device_types:
            results = db.select_all(device_type, {'room': room, 'von': von, 'bis': bis})

            for item in results:
                #print(item)

                current_item_value = room_data_dict[str(item['DEVICE_ID'])+str(item['DEVICE_TYPE'])]

                percentage.append({'type': item['DEVICE_TYPE'],'id': item['DEVICE_ID'],
                                   'location': room, 'name': item['DEVICE_NAME'], 'usage': float(item['POWER_USAGE'])+current_item_value})

        data['usagepercentage'] = percentage

        data['usagecourse'] = self.get_energy_course(user, room, von, bis, db)
        data['usagecourse'].append({'date': time.strftime('%d.%m.%Y'), 'usage': room_total_usage})

        return data

    def get_device_energy_data(self, user, room , devicetype, deviceid, von, bis, db: Database = None):
        if db is None:
            db = Database()
        data = {}
        result = db.select_one("SELECT SUM(POWER_USAGE) AS POWER_USAGE FROM DEVICE_ENERGY_DATA WHERE DEVICE_TYPE = :type AND DEVICE_ID = :id AND DATE >= :von AND DATE <= :bis",
                    {'type': devicetype, 'id': deviceid, 'von':von, 'bis': bis})

        data['totalusage'] = float(result['POWER_USAGE'])
        data['cost'] = self.get_power_cost(user, db)

        data['usagcourse'] = self.get_device_energy_course(user, room, devicetype, deviceid, von, bis, db)

        return data

    def get_device_energy_course(self, user, room, type, id, von, bis, db: Database = None):
        if db is None:
            db = Database()
        course = []

        results = db.select_all("SELECT POWER_USAGE, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM DEVICE_ENERGY_DATA WHERE DEVICE_TYPE = :type AND DEVICE_ID = :id AND DATE >= :von AND DATE <= :bis ORDER BY DATE ASC",
            {'type': type, 'id': id, 'von': von, 'bis': bis})

        for item in results:
            course.append({'date': item['DATETIME'], 'usage': item['POWER_USAGE']})

        return course

    def get_energy_course(self, user, room, von, bis, db: Database = None):
        if db is None:
            db = Database()
        course = []

        if (room is not None and room != ""):
            results = db.select_all("""SELECT POWER_USAGE, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM ROOM_ENERGY_DATA 
            WHERE ROOM_ID = :room AND DATE >= :von AND DATE <= :bis ORDER BY DATE ASC""",
                                          {'room': room, 'von': von, 'bis': bis})
        else:
            results = db.select_all("""SELECT SUM(POWER_USAGE) as POWER_USAGE, strftime(\"%d.%m.%Y\", DATE) as DATETIME 
            FROM ROOM_ENERGY_DATA WHERE DATE >= :von AND DATE <= :bis GROUP BY DATE ORDER BY DATE ASC""",
                                          {'von': von, 'bis': bis})

        for item in results:
            course.append({'date': item['DATETIME'], 'usage': item['POWER_USAGE']})

        return course

    def set_power_cost(self, user, cost, db: Database = None):
        if db is None:
            db = Database()
        #TODO set power cost

    def get_power_cost(self, user, db: Database = None):
        if db is None:
            db = Database()
        power_cost = 0.3

        return round(power_cost, 2)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.MediaCenterManager import MediaCenterManager

ACTION_KEY_GET_MEDIA_CENTERS = "getmediacenters"
ACTION_KEY_CONTROL = "mediacentercontrol"
ACTION_KEY_SEND_TEXT = "mediacentersendtext"
ACTION_KEY_MUSIC = "mediacentermusic"
ACTION_KEY_ARTISTS = "mediacenterartists"
ACTION_KEY_ALBUMS = "mediacenteralbums"
ACTION_KEY_MUSIC_GENRES = "mediacentermusicgenres"
ACTION_KEY_TV_SHOWS = "mediacentertvshows"
ACTION_KEY_TV_SHOW_SEASONS = "mediacentertvshowseasons"
ACTION_KEY_TV_SHOW_EPISODES = "mediacentertvshowepisodes"
ACTION_KEY_MOVIES = "mediacentermovies"
ACTION_KEY_MOVIE_GENRES = "mediacentermoviegenres"
ACTION_KEY_PLAYING = "mediacenterplaying"

class MediaCenterAPIModule(APIModule):
    def __init__(self):
        super(MediaCenterAPIModule, self).__init__()
        self.manager = MediaCenterManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_MEDIA_CENTERS: self.get_media_centers,
            ACTION_KEY_CONTROL: self.control,
            ACTION_KEY_SEND_TEXT: self.send_text,
            ACTION_KEY_MUSIC: self.get_music,
            ACTION_KEY_ARTISTS: self.get_artists,
            ACTION_KEY_ALBUMS: self.get_albums,
            ACTION_KEY_MUSIC_GENRES: self.get_music_genres,
            ACTION_KEY_TV_SHOWS: self.get_tv_shows,
            ACTION_KEY_TV_SHOW_SEASONS: self.get_tv_show_seasons,
            ACTION_KEY_TV_SHOW_EPISODES: self.get_tv_show_episodes,
            ACTION_KEY_MOVIES: self.get_movies,
            ACTION_KEY_MOVIE_GENRES: self.get_movie_genres,
            ACTION_KEY_PLAYING: self.get_playing
        }

        return mappings

    def get_media_centers(self, user, request, db):
        data = self.manager.get_media_centers(user, db)
        return Status(type=STATUS_OK, data={'mediacenters': data})

    def control(self, user, request, db):
        data = self.manager.media_remote_action(request['type'], request['id'], request['remoteaction'], db)
        return Status(type=STATUS_OK, data=data)

    def send_text(self, user, request, db):
        data = self.manager.send_text(request['type'], request['id'], request['text'], db)
        return Status(type=STATUS_OK, data=data)

    def get_music(self, user, request, db):
        data = self.manager.get_music(request['type'], request['id'],
                                      request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'music': data})

    def get_artists(self, user, request, db):
        data = self.manager.get_artists(request['type'], request['id'],
                                        request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'artists': data})

    def get_albums(self, user, request, db):
        data = self.manager.get_albums(request['type'], request['id'],
                                       request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'albums': data})

    def get_music_genres(self, user, request, db):
        data = self.manager.get_music_genres(request['type'], request['id'],
                                             request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'genres': data})

    def get_tv_shows(self, user, request, db):
        data = self.manager.get_shows(request['type'], request['id'],
                                      request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'shows': data})

    def get_tv_show_seasons(self, user, request, db):
        data = self.manager.get_show_seasons(request['type'], request['id'], request['limit'],
                                             request['offset'], request['showid'], db)
        return Status(type=STATUS_OK, data={'seasons': data})

    def get_tv_show_episodes(self, user, request, db):
        data = self.manager.get_show_episodes(request['type'], request['id'], request['limit'],
                                              request['offset'], request['showid'],
                                              request['seasonid'], db)
        return Status(type=STATUS_OK, data={'episodes': data})

    def get_movies(self, user, request, db):
        data = self.manager.get_movies(request['type'], request['id'],
                                       request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'movies': data})

    def get_movie_genres(self, user, request, db):
        data = self.manager.get_movie_genres(request['type'], request['id'],
                                             request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'genres': data})

    def get_playing(self, user, request, db):
        data = self.manager.get_playing(request['type'], request['id'], db)
        return Status(type=STATUS_OK, data=data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback

from Homevee.APIModule.APIKeyAPIModule import APIKeyAPIModule
from Homevee.APIModule.AutomationAPIModule import AutomationAPIModule
from Homevee.APIModule.BlindsAPIModule import BlindsAPIModule
from Homevee.APIModule.CalendarAPIModule import CalendarAPIModule
from Homevee.APIModule.ChatAPIModule import ChatAPIModule
from Homevee.APIModule.CustomVoiceCommandAPIModule import CustomVoiceCommandAPIModule
from Homevee.APIModule.DashboardAPIModule import DashboardAPIModule
from Homevee.APIModule.DeviceAPIModule import DeviceAPIModule
from Homevee.APIModule.DimmerAPIModule import DimmerAPIModule
from Homevee.APIModule.EnergyDataAPIModule import EnergyDataAPIModule
from Homevee.APIModule.EventAPIModule import EventAPIModule
from Homevee.APIModule.GPSDataAPIModule import GPSDataAPIModule
from Homevee.APIModule.GatewayAPIModule import GatewayAPIModule
from Homevee.APIModule.GraphDataAPIModule import GraphDataAPIModule
from Homevee.APIModule.HeatingSchemeAPIModule import HeatingSchemeAPIModule
from Homevee.APIModule.HomeBudgetAPIModule import HomeBudgetAPIModule
from Homevee.APIModule.ImageClassifierAPIModule.ARControlAPIModule import ARControlAPIModule
from Homevee.APIModule.ImageClassifierAPIModule.PeopleClassifierAPIModule import PeopleClassifierAPIModule
from Homevee.APIModule.LoginAPIModule import LoginAPIModule
from Homevee.APIModule.MQTTConnectorAPIModule import MQTTConnectorAPIModule
from Homevee.APIModule.MediaCenterAPIModule import MediaCenterAPIModule
from Homevee.APIModule.NutritionDataAPIModule import NutritionDataAPIModule
from Homevee.APIModule.PersonAPIModule import PersonAPIModule
from Homevee.APIModule.PlaceAPIModule import PlaceAPIModule
from Homevee.APIModule.RBGLightAPIModule import RGBLightAPIModule
from Homevee.APIModule.RFIDTagAPIModule import RFIDTagAPIModule
from Homevee.APIModule.RemoteControlAPIModule import RemoteControlAPIModule
from Homevee.APIModule.RoomAPIModule import RoomAPIModule
from Homevee.APIModule.RoomDataAPIModule import RoomDataAPIModule
from Homevee.APIModule.SceneAPIModule import SceneAPIModule
from Homevee.APIModule.SensorDataAPIModule import SensorDataAPIModule
from Homevee.APIModule.ShoppingListAPIModule import ShoppingListAPIModule
from Homevee.APIModule.SmartSpeakerAPIModule import SmartSpeakerAPIModule
from Homevee.APIModule.SystemInfoAPIModule import SystemInfoAPIModule
from Homevee.APIModule.TVPlanAPIModule import TVPlanAPIModule
from Homevee.APIModule.ThermostatAPIModule import ThermostatAPIModule
from Homevee.APIModule.UpdaterAPIModule import UpdaterAPIModule
from Homevee.APIModule.UserAPIModule import UserAPIModule
from Homevee.APIModule.VoiceAPIModule import VoiceAPIModule
from Homevee.APIModule.WakeOnLanAPIModule import WakeOnLanAPIModule
from Homevee.APIModule.WeatherAPIModule import WeatherAPIModule
from Homevee.Exception import NoPermissionException
from Homevee.Helper import translations
# from Functions import people_classifier, ar_control
# from Functions.people_classifier import *
from .Manager.UserManager import User
from .VoiceAssistant import *
from .VoiceAssistant.VoiceReplaceManager import *

FUNCTION_MAPPINGS = {}

class API:
    def __init__(self):
        return

    def process_data(self, data: dict, db: Database) -> dict:
        """
        Process the request
        :param data: the data dict
        :param db: the database connection
        :return: the response data dict
        """
        try:
            username = data['username']
            password = data['password']

            user = User.load_username_from_db(username, db)
            verified = user.verify(password)

            if 'language' in data and data['language'] is not None:
                language = data['language']
            else:
                language = translations.LANGUAGE

            if not verified:
                return Status(type=STATUS_WRONG_DATA).get_dict()

            if(data['action'] in FUNCTION_MAPPINGS):
                #print("########################################")
                #print("####ANTWORT DER NEUEN HANDLE REQUEST####")
                #print("########################################")
                print(json.dumps(data))
                response = self.handle_request(user, data, db).get_dict()
                print(response)
                #print("########################################")
                #print("########################################")
                return response
            else:
                return Status(type=STATUS_NO_SUCH_ACTION).get_dict()
        except NoPermissionException as e:
            if Logger.IS_DEBUG:
                traceback.print_exc()
            return Status(type=STATUS_NO_PERMISSION).get_dict()
        except Exception as e:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return Status(type=STATUS_ERROR).get_dict()

    def create_function_mappings(self):
        modules = [LoginAPIModule(), RoomAPIModule(), UserAPIModule(), RoomDataAPIModule(), GraphDataAPIModule(),
                   VoiceAPIModule(), ChatAPIModule(), SystemInfoAPIModule(), HomeBudgetAPIModule(), WeatherAPIModule(),
                   RemoteControlAPIModule(), DashboardAPIModule(), CalendarAPIModule(), GPSDataAPIModule(),
                   PlaceAPIModule(), NutritionDataAPIModule(), PersonAPIModule(), RFIDTagAPIModule(), EventAPIModule(),
                   GatewayAPIModule(), TVPlanAPIModule(), EnergyDataAPIModule(), UpdaterAPIModule(), HeatingSchemeAPIModule(),
                   SensorDataAPIModule(), ShoppingListAPIModule(), CustomVoiceCommandAPIModule(), SceneAPIModule(),
                   AutomationAPIModule(), RGBLightAPIModule(), ARControlAPIModule(), PeopleClassifierAPIModule(),
                   BlindsAPIModule(), DimmerAPIModule(), WakeOnLanAPIModule(), MQTTConnectorAPIModule(),
                   SmartSpeakerAPIModule(), ThermostatAPIModule(), APIKeyAPIModule(), DeviceAPIModule(),
                   MediaCenterAPIModule()]
        for module in modules:
            module_mappings = module.get_function_mappings()
            for action_key in module_mappings:
                if action_key in FUNCTION_MAPPINGS:
                    raise RuntimeError("Action-Key already added to FUNCTION_MAPPINGS")
                FUNCTION_MAPPINGS[action_key] = module_mappings[action_key]

    def get_function_mappings(self):
        return FUNCTION_MAPPINGS

    def handle_request(self, user: User, request: dict, db: Database = None) -> Status:
        if db is None:
            db = Database()
        action = request['action']
        if action in FUNCTION_MAPPINGS:
            return FUNCTION_MAPPINGS[action](user, request, db)
        else:
            return Status(type=STATUS_NO_SUCH_ACTION)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.APIKeyManager import APIKeyManager

ACTION_KEY_GET_API_KEY_DATA = "getapikeydata"
ACTION_KEY_SET_API_KEY = "setapikey"

class APIKeyAPIModule(APIModule):
    def __init__(self):
        super(APIKeyAPIModule, self).__init__()
        self.api_key_manager = APIKeyManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_API_KEY_DATA: self.get_api_key_data,
            ACTION_KEY_SET_API_KEY: self.set_api_key
        }

        return mappings

    def get_api_key_data(self, user, request, db):
        data = self.api_key_manager.get_all_api_key_data(user, db)
        return Status(type=STATUS_OK, data={'apikeys': data})

    def set_api_key(self, user, request, db):
        data = self.api_key_manager.set_api_key(user, request['service'], request['apikey'], db)
        return Status(type=STATUS_OK)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item import APIKey, Item
from Homevee.Item.APIKey import APIKey
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database

class APIKeyManager():
    def get_all_api_key_data(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        services = [
            {'servicename': 'Open Weather Map',
             'description': 'Open Weather Map stellt Wetterdaten und -vorhersagen bereit.', 'registerkeyurl': '',
             'servicelogourl': 'https://openweathermap.org/themes/openweathermap/assets/vendor/owm/img/logo_OpenWeatherMap_orange.svg'},
            {'servicename': 'IMDB', 'description': 'IMDB ist eine internationale Filmdatenbank', 'registerkeyurl': '',
             'servicelogourl': 'http://ia.media-imdb.com/images/G/01/imdb/images/mobile/imdb-logo-responsive@2-868559777._CB514893749_.png'}
        ]

        output = []

        for service in services:
            api_key = self.get_api_key(service['servicename'], db)

            if api_key is not None:
                service['key'] = api_key['key']
                output.append(service)

        return output

    def get_api_key(self, service_name, db: Database = None):
        if db is None:
            db = Database()
        api_key = APIKey.find_by_name(service_name, db)

        if api_key is not None:
            return api_key.get_dict()
        else:
            return None

    def set_api_key(self, user, id, service, api_key, db: Database = None):
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        if db is None:
            db = Database()

        api_key = Item.load_from_db(APIKey, id, db)

        if api_key is None:
            api_key = APIKey(service, api_key)

        return api_key.api_save(db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Helper import Logger
from Homevee.Item.User import User
from Homevee.Manager.SensorDataManager import SensorDataManager
from Homevee.Utils.Database import Database


class ConditionManager:
    def __init__(self):
        return

    def conditions_true(self, condition_data, db: Database = None):
        if db is None:
            db = Database()
        #Format: {'and':[{'and':[{'action':'time', 'comparator':'greater', 'value':'08:00'}, {'action':'time', 'comparator':'less', 'value':'20:00'}]},
        # {'and':[{}, {}]}]}

        if('and' in condition_data):
            for condition in condition_data['and']:
                if not self.conditions_true(condition, db):
                    return False
                return True
        if(isinstance(condition_data, list)):
            if(len(condition_data) == 0):
                return True
            for condition in condition_data:
                if not self.conditions_true(condition, db):
                    return False
                return True
        elif('or' in condition_data):
            for condition in condition_data['or']:
                if self.conditions_true(condition, db):
                    return True
            return False
        else:
            #check single conditions
            return self.check_single_condition(condition_data)

    def check_single_condition(self, condition_data, db: Database = None):
        if db is None:
            db = Database()
        #{'action': 'timeperiod', 'from': '08:00', 'to': '20:00'}
        if(condition_data['type'] == "timeperiod"):
            return self.is_in_time_period(condition_data)
        #{'action': 'weekdays', 'days': [5, 6]}
        elif(condition_data['type'] == "weekdays"):
            return datetime.datetime.today().weekday() in condition_data['days']
        #{'action': 'sensor_value', 'type': 'MQTT Sensor', 'id': '1', 'comparator': '>', 'value': 28}
        elif(condition_data['type'] == "sensorvalue"):
            value = SensorDataManager().get_sensor_value(condition_data['devicetype'], condition_data['deviceid'], False)
            Logger.log(value)
            return self.check_value(condition_data['comparator'], value, condition_data['value'])
        #{'action': 'user_at_home', 'user': 'sascha', 'at_home': False}
        elif(condition_data['type'] == "user_at_home"):
            user = User.load_username_from_db(condition_data['username'])

            if(condition_data['at_home']):
                return user.at_home
            elif(not condition_data['at_home']):
                return not user.at_home
        return False

    def check_value(self, comparator, is_value, value_condition):
        if(comparator == ">"):
            return float(is_value) > float(value_condition)
        elif(comparator == ">="):
            return float(is_value) >= float(value_condition)
        elif(comparator == "<"):
            return float(is_value) < float(value_condition)
        elif(comparator == "<="):
            return float(is_value) <= float(value_condition)
        elif(comparator == "=="):
            return float(is_value) == float(value_condition)
        elif(comparator == "!="):
            return float(is_value) != float(value_condition)

    def is_in_time_period(self, condition_data):
        today = datetime.datetime.today()

        from_hour, from_minute = condition_data['from'].split(":")

        to_hour, to_minute = condition_data['to'].split(":")

        from_hour = int(from_hour)
        from_minute = int(from_minute)
        to_hour = int(to_hour)
        to_minute = int(to_minute)

        if (today.hour == from_hour):
            if (today.minute >= from_minute):
                if (today.hour == to_hour):
                    if (today.minute <= to_minute):
                        return True
                elif (today.hour < to_hour):
                    return True
        elif (today.hour >= from_hour):
            if (today.hour == to_hour):
                if (today.minute <= to_minute):
                    return True
            elif (today.hour < to_hour):
                return True
        else:
            return False


from Homevee.APIModule import APIModule
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Manager.RoomManager import RoomManager

ACTION_KEY_LOGIN = "login"

class LoginAPIModule(APIModule):
    def __init__(self):
        super(LoginAPIModule, self).__init__()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_LOGIN: self.do_login
        }

        return mappings

    def do_login(self, user, request, db):
        rooms = RoomManager().get_rooms(user, db)
        remote_id = db.get_server_data("REMOTE_ID")
        access_token = "ACCESS_TOKEN"
        data = {'remote_id': remote_id, 'access_token': access_token, 'rooms': Room.list_to_dict(rooms)}
        return Status(type=STATUS_OK, data=data)


import json
import urllib.error
import urllib.parse
import urllib.request

from Homevee.Exception import DatabaseSaveFailedException, NoPermissionException
from Homevee.Item.User import User
from Homevee.Utils.Database import Database


class RemoteControlManager():
    def __init__(self):
        return

    def remote_control_enabled(self, db):
        REMOTE_CONTROL_ENABLED = db.get_server_data("USE_REMOTE_CONTROL", db)
        return REMOTE_CONTROL_ENABLED == "true"

    def set_remote_control_enabled(self, user: User, enabled, db: Database = None):
        if db is None:
            db = Database()
        if (not user.has_permission("admin")):
            return {'result': 'noadmin'}
        try:
            db.set_server_data("USE_REMOTE_CONTROL", enabled)
        except:
            raise DatabaseSaveFailedException

    def save_remote_data(self, user, remote_id, linked_account, db: Database = None):
        if db is None:
            db = Database()
        if (not user.has_permission("admin")):
            raise NoPermissionException

        try:
            db.set_server_data("REMOTE_ID", remote_id)
            db.set_server_data("REMOTE_LINKED_ACCOUNT", linked_account)
        except:
            raise DatabaseSaveFailedException

    def load_remote_data(self, user, db: Database = None):
        if db is None:
            db = Database()
        if (not user.has_permission("admin")):
            return {'result': 'noadmin'}

        remote_id = db.get_server_data("REMOTE_ID")
        access_token = db.get_server_data("REMOTE_ACCESS_TOKEN")
        linked_account = db.get_server_data("REMOTE_LINKED_ACCOUNT")

        # check if account is still linked

        # load premium membership
        url = "https://cloud.homevee.de/server-api.php?action=getpremiumuntil&remoteid=" + remote_id + "&accesstoken=" + access_token
        contents = urllib.request.urlopen(url).read()
        data = json.loads(contents)
        is_premium = None

        try:
            is_premium = data['is_premium']
            premium_until = data['premium_until']
        except:
            premium_until = None

        return {'remote_id': remote_id,
                'access_token': access_token,
                'linked_account': linked_account, 'premium_until': premium_until, 'is_premium': is_premium,
                'remote_control_enabled': self.remote_control_enabled(db)}

    def connect_remote_id_with_account(self, user, account_name, account_secret, db: Database = None):
        if db is None:
            db = Database()
        if (not user.has_permission("admin")):
            return {'result': 'noadmin'}


        remote_id = db.get_server_data("REMOTE_ID")
        url = "https://Homevee.de/connect-remote-id.php?remote_id=" + remote_id + "&account_name=" \
              + account_name + "&account_secret=" + account_secret
        response = urllib.request.urlopen(url).read()

        response = response.decode('utf-8')

        if response == "ok":
            db.set_server_data("REMOTE_LINKED_ACCOUNT", account_name)
            return True
        else:
            return False


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.AutomationManager import AutomationManager

ACTION_KEY_GET_AUTOMATION_RULES = "getautomationrules"
ACTION_KEY_ADD_EDIT_AUTOMATION_RULE = "addeditautomationrule"
ACTION_KEY_DELETE_AUTOMATION_RULE = "deleteautomationrule"

class AutomationAPIModule(APIModule):
    def __init__(self):
        super(AutomationAPIModule, self).__init__()
        self.automation_manager = AutomationManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_AUTOMATION_RULES: self.get_automation_rules,
            ACTION_KEY_ADD_EDIT_AUTOMATION_RULE: self.add_edit_automation_rule,
            ACTION_KEY_DELETE_AUTOMATION_RULE: self.delete_automation_rule
        }

        return mappings

    def get_automation_rules(self, user, request, db):
        data = self.automation_manager.get_automations(user, request['location'], db)
        return Status(type=STATUS_OK, data={'rules': data})

    def add_edit_automation_rule(self, user, request, db):
        self.automation_manager.add_edit_automation_rule(user, request['location'], request['id'],
                                                                    request['name'],
                                                                    request['triggerdata'], request['conditiondata'],
                                                                    request['actiondata'],
                                                                    request['isactive'], db)
        return Status(type=STATUS_OK)

    def delete_automation_rule(self, user, request, db):
        self.automation_manager.delete_automation_rule(user, request['id'], db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.DimmerManager import DimmerManager

ACTION_KEY_SET_DIMMER = "setdimmer"

class DimmerAPIModule(APIModule):
    def __init__(self):
        super(DimmerAPIModule, self).__init__()
        self.dimmer_manager = DimmerManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_SET_DIMMER: self.set_dimmer
        }

        return mappings

    def set_dimmer(self, user, request, db):
        self.dimmer_manager.set_dimmer(user, request['type'], request['id'], request['value'], db)
        return Status(type=STATUS_OK)


from abc import abstractmethod

from Homevee.APIModule.ImageClassifierAPIModule import ImageClassifierAPIModule
from Homevee.Manager.ControlManager.ImageClassifierManager.ARControlManager import ARControlManager

ACTION_KEY_CLASSIFY_IMAGE = "arcontrol"
ACTION_KEY_GET_CLASS = "getarcontrolclasses"
ACTION_KEY_SAVE_CLASSES = "savearcontrolclasses"
ACTION_KEY_START_TRAINING = "startarcontroltraining"
ACTION_KEY_CHANGE_IMAGE_CLASS = "changearcontrolimageclass"
ACTION_KEY_GET_CLASS_IMAGES = "getarcontrolclassimages"
ACTION_KEY_GET_IMAGE = "getarcontrolimage"
ACTION_KEY_GET_PERFORMANCE_SETTINGS = "getarcontrolperformancesettings"
ACTION_KEY_SET_PERFORMANCE_SETTINGS = "setarcontrolperformancesettings"
ACTION_KEY_UPLOAD_IMAGES = "uploadarcontrolimages"
ACTION_KEY_DELETE_IMAGES = "deletearcontrolimages"

class ARControlAPIModule(ImageClassifierAPIModule):
    def __init__(self):
        manager = ARControlManager()
        super(ARControlAPIModule, self).__init__(manager)
        return

    @abstractmethod
    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_CLASSIFY_IMAGE: self.classify_image,
            ACTION_KEY_GET_CLASS: self.get_classes,
            ACTION_KEY_SAVE_CLASSES: self.save_class,
            ACTION_KEY_START_TRAINING: self.start_training,
            ACTION_KEY_CHANGE_IMAGE_CLASS: self.change_image_class,
            ACTION_KEY_GET_CLASS_IMAGES: self.get_class_images,
            ACTION_KEY_GET_IMAGE: self.get_image,
            ACTION_KEY_GET_PERFORMANCE_SETTINGS: self.get_performance_settings,
            ACTION_KEY_SET_PERFORMANCE_SETTINGS: self.set_performance_settings,
            ACTION_KEY_UPLOAD_IMAGES: self.upload_images,
            ACTION_KEY_DELETE_IMAGES: self.delete_images
        }
        return mappings


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Room import Room
from Homevee.Utils.Database import Database

class SceneManager():
    def __init__(self):
        return

    def get_all_scenes(self, user, db: Database = None):
        if db is None:
            db = Database()

        results = db.select_all("SELECT * FROM ROOMS", None)

        rooms = []

        for room in results:
            if not user.has_permission(room['LOCATION']):
                continue

            scenes = self.get_scenes(user, room['LOCATION'], db)

            if(len(scenes) is not 0):
                room_item = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'], 'scenes': scenes}
                rooms.append(room_item)

        return rooms

    def get_scenes(self, user, location, db: Database = None):
        if db is None:
            db = Database()
        scenes = []

        if isinstance(location, Room):
            location = location.id

        if user.has_permission(location):
            items = db.select_all("SELECT * FROM SCENES WHERE ROOM = :location",{'location': location})

            for item in items:
                scenes.append({'id': item['ID'], 'name': item['NAME'], 'action_data': item['ACTION_DATA'],
                              'location': item['ROOM']})

        return scenes

    def add_edit_scene(self, user, id, name, location, action_data, db: Database = None):
        if db is None:
            db = Database()
        add_new = (id == None or id == "" or id == "-1")

        if(add_new):
            db.insert("INSERT INTO SCENES (NAME, ROOM, ACTION_DATA) VALUES (:name, :room, :actions)",
                        {'name': name, 'room': location, 'actions': action_data})
        else:
            db.update("UPDATE SCENES SET NAME = :name, ROOM = :location, ACTION_DATA = :actions WHERE ID = :id",
                {'name': name, 'location': location, 'actions': action_data, 'id': id})

    def delete_scene(self, user, id, db: Database = None):
        if db is None:
            db = Database()

        db.delete("DELETE FROM SCENES WHERE ID = :id", {'id': id})